@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 2003 and 2004 by the Free Software Foundation.          *
+ * Copyright 2003, 2004 and 2005 by the Free Software Foundation.    *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
  * modify it under the terms of the GNU General Public License as    *
@@ -15,7 +15,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -58,8 +58,8 @@ struct break_in_data {
 };
 
 #define MAX_BREAK_INS 50
-struct break_in_data break_in_list[MAX_BREAK_INS];
-int num_break_ins;
+static struct break_in_data break_in_list[MAX_BREAK_INS];
+static int num_break_ins;
 
 
 /* Adds all empty intersections that have two goal neighbors to the goal. */
@@ -218,7 +218,7 @@ compute_smaller_goal(int owner, int color_to_move,
 static int
 break_in_goal_from_str(int str, char goal[BOARDMAX],
     		      int *num_non_territory, int non_territory[BOARDMAX],
-    		      int color_to_move)
+    		      int color_to_move, int info_pos)
 {
   int move = NO_MOVE;
   int saved_move = NO_MOVE;
@@ -227,9 +227,9 @@ break_in_goal_from_str(int str, char goal[BOARDMAX],
 
   /* When blocking off, we use a somewhat smaller goal area. */
   if (color_to_move == board[str])
-    compute_connection_distances(str, NO_MOVE, 3.01, &conn, 1);
+    compute_connection_distances(str, NO_MOVE, FP(3.01), &conn, 1);
   else
-    compute_connection_distances(str, NO_MOVE, 2.81, &conn, 1);
+    compute_connection_distances(str, NO_MOVE, FP(2.81), &conn, 1);
 
   sort_connection_queue_tail(&conn);
   expand_connection_queue(&conn);
@@ -253,21 +253,29 @@ break_in_goal_from_str(int str, char goal[BOARDMAX],
     int k;
     int save_num = *num_non_territory;
     int affected_size = 0;
-    float cut_off_distance = 3.5;
+    int cut_off_distance = FP(3.5);
     if (ON_BOARD(move) && goal[move]) {
       non_territory[(*num_non_territory)++] = move;
-      DEBUG(DEBUG_TERRITORY, "Erasing territory at %1m -a.\n", move);
+      if (info_pos)
+	DEBUG(DEBUG_TERRITORY, "%1m: Erasing territory at %1m -a.\n",
+	      info_pos, move);
+      else
+	DEBUG(DEBUG_TERRITORY, "Erasing territory at %1m -a.\n", move);
     }
 
     for (k = 0; k < conn.queue_end; k++) {
       int pos = conn.queue[k];
-      if (conn.distances[pos] > cut_off_distance + 0.31)
+      if (conn.distances[pos] > cut_off_distance + FP(0.31))
 	break;
       if (goal[pos]
 	  && (!ON_BOARD(conn.coming_from[pos])
 	      || !goal[conn.coming_from[pos]])) {
 	non_territory[(*num_non_territory)++] = pos;
-	DEBUG(DEBUG_TERRITORY, "Erasing territory at %1m -b.\n", pos);
+	if (info_pos)
+	  DEBUG(DEBUG_TERRITORY, "%1m: Erasing territory at %1m -b.\n",
+		info_pos, pos);
+	else
+	  DEBUG(DEBUG_TERRITORY, "Erasing territory at %1m -b.\n", pos);
 	if (conn.distances[pos] < cut_off_distance)
 	  cut_off_distance = conn.distances[pos];
       }
@@ -312,7 +320,7 @@ break_in_goal_from_str(int str, char goal[BOARDMAX],
 
 static void
 break_in_goal(int color_to_move, int owner, char goal[BOARDMAX],
-    	      struct influence_data *q, int store)
+    	      struct influence_data *q, int store, int info_pos)
 {
   struct connection_data conn;
   int k;
@@ -322,14 +330,14 @@ break_in_goal(int color_to_move, int owner, char goal[BOARDMAX],
   int num_non_territory = 0;
   int candidate_strings[MAX_TRIES];
   int candidates = 0;
-  float min_distance = 5.0;
+  int min_distance = FP(5.0);
 
   DEBUG(DEBUG_BREAKIN,
         "Trying to break (%C to move) %C's territory ", color_to_move, owner);
   if (debug & DEBUG_BREAKIN)
     goaldump(goal);
   /* Compute nearby fields of goal. */
-  init_connection_data(intruder, goal, NO_MOVE, 3.01, &conn, 1);
+  init_connection_data(intruder, goal, NO_MOVE, FP(3.01), &conn, 1);
   k = conn.queue_end;
   spread_connection_distances(intruder, &conn);
   sort_connection_queue_tail(&conn);
@@ -340,7 +348,7 @@ break_in_goal(int color_to_move, int owner, char goal[BOARDMAX],
   memset(used, 0, BOARDMAX);
   for (; k < conn.queue_end; k++) {
     int pos = conn.queue[k];
-    if (conn.distances[pos] > min_distance + 1.001)
+    if (conn.distances[pos] > min_distance + FP(1.001))
       break;
     if (board[pos] == intruder
 	&& influence_considered_lively(q, pos)) {
@@ -373,7 +381,7 @@ break_in_goal(int color_to_move, int owner, char goal[BOARDMAX],
   for (k = 0; k < candidates; k++) {
     int move = break_in_goal_from_str(candidate_strings[k], goal,
   		                     &num_non_territory, non_territory,
-				     color_to_move);
+				     color_to_move, info_pos);
     if (store && ON_BOARD(move) && num_break_ins < MAX_BREAK_INS) {
       /* Remember the move as a possible move candidate for later. */
       break_in_list[num_break_ins].str = candidate_strings[k];
@@ -396,7 +404,8 @@ break_in_goal(int color_to_move, int owner, char goal[BOARDMAX],
  * later gets used to generate move reasons).
  */
 void
-break_territories(int color_to_move, struct influence_data *q, int store)
+break_territories(int color_to_move, struct influence_data *q, int store,
+    		  int info_pos)
 {
   struct moyo_data territories;
   int k;
@@ -422,7 +431,8 @@ break_territories(int color_to_move, struct influence_data *q, int store)
 
     if (color_to_move == OTHER_COLOR(territories.owner[k]))
       enlarge_goal(goal);
-    break_in_goal(color_to_move, territories.owner[k], goal, q, store);
+    break_in_goal(color_to_move, territories.owner[k], goal, q, store,
+		  info_pos);
   }
 }
 
