@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -390,7 +390,7 @@ atari_atari_blunder_size(int color, int move, char defense_moves[BOARDMAX],
 			 const char safe_stones[BOARDMAX])
 {
   int apos;
-  int defense_point = NO_MOVE, after_defense_point = NO_MOVE;
+  int defense_point = NO_MOVE;
   int aa_val, after_aa_val;
   int other = OTHER_COLOR(color);
   char defense_points[BOARDMAX];
@@ -439,7 +439,6 @@ atari_atari_blunder_size(int color, int move, char defense_moves[BOARDMAX],
      * works. What we really want is to get the proper defense move
      * into (*defense).
      */
-    after_defense_point = defense_point;
     forbidden[apos] = 1;
     last_forbidden = apos;
     aa_val = do_atari_atari(other, &apos, &defense_point, NULL,
