@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -139,54 +139,9 @@ make_dragons(int stop_before_owl)
   analyze_false_eye_territory();
 
   /* Now we compute the genus. */
-#if 1
-
   for (d = 0; d < number_of_dragons; d++)
     compute_dragon_genus(dragon2[d].origin, &dragon2[d].genus, NO_MOVE);
 
-#else
-
-  {
-    int dr;
-
-    for (str = BOARDMIN; str < BOARDMAX; str++) {
-      if (!ON_BOARD(str))
-	continue;
-
-      if (black_eye[str].color == BLACK
-	  && black_eye[str].origin == str
-	  && find_eye_dragons(black_eye[str].origin, black_eye,
-			      BLACK, &dr, 1) == 1) {
-
-	ASSERT1(board[dr] == BLACK, dr);
-	TRACE("eye at %1m found for dragon at %1m--augmenting genus\n",
-	      str, dr);
-	if (eye_move_urgency(&black_eye[str].value)
-	    > eye_move_urgency(&DRAGON2(dr).genus))
-	  DRAGON2(dr).heye = black_eye[str].defense_point;
-	add_eyevalues(&DRAGON2(dr).genus, &black_eye[str].value,
-		      &DRAGON2(dr).genus);
-      }
-
-      if (white_eye[str].color == WHITE
-	  && white_eye[str].origin == str
-	  && find_eye_dragons(white_eye[str].origin, white_eye,
-			      WHITE, &dr, 1) == 1) {
-
-	ASSERT1(board[dr] == WHITE, dr);
-	TRACE("eye at %1m found for dragon at %1m--augmenting genus\n",
-	      str, dr);
-	if (eye_move_urgency(&white_eye[str].value)
-	    > eye_move_urgency(&DRAGON2(dr).genus))
-	  DRAGON2(dr).heye = white_eye[str].defense_point;
-	add_eyevalues(&DRAGON2(dr).genus, &white_eye[str].value,
-		      &DRAGON2(dr).genus);
-      }
-    }
-  }
-
-#endif
-
   /* Compute the escape route measure. */
   for (str = BOARDMIN; str < BOARDMAX; str++)
     if (IS_STONE(board[str]) && dragon[str].origin == str)
@@ -400,7 +355,8 @@ make_dragons(int stop_before_owl)
 	
 	if (level >= 8
 	    && !disable_threat_computation
-	    && owl_threats) {
+	    && (owl_threats 
+		|| thrashing_stone[str])) {
 	  if (acode && !dcode && DRAGON2(str).owl_attack_point != NO_MOVE) {
 	    if (owl_threaten_defense(str, &defense_point,
 				     &second_defense_point)) {
@@ -493,7 +449,7 @@ make_dragons(int stop_before_owl)
   lively_white_dragons = 0;
   lively_black_dragons = 0;
   for (d = 0; d < number_of_dragons; d++)
-    if (DRAGON(d).crude_status != DEAD) {
+    if (DRAGON(d).status != DEAD) {
       if (DRAGON(d).color == WHITE)
         lively_white_dragons++;
       else
@@ -571,8 +527,6 @@ eye_computations()
       DEBUG(DEBUG_EYES, "Black eyespace at %1m: %s\n", str,
 	    eyevalue_to_string(&value));
       black_eye[str].value = value;
-      black_eye[str].attack_point = attack_point;
-      black_eye[str].defense_point = defense_point;
       propagate_eye(str, black_eye);
     }
     
@@ -586,8 +540,6 @@ eye_computations()
       DEBUG(DEBUG_EYES, "White eyespace at %1m: %s\n", str,
 	    eyevalue_to_string(&value));
       white_eye[str].value = value;
-      white_eye[str].attack_point = attack_point;
-      white_eye[str].defense_point = defense_point;
       propagate_eye(str, white_eye);
     }
   }
@@ -1270,12 +1222,12 @@ compute_dragon_influence()
   set_strength_data(BLACK, safe_stones, strength);
   compute_influence(BLACK, safe_stones, strength, &initial_black_influence,
                     NO_MOVE, "initial black influence, dragons known");
-  break_territories(BLACK, &initial_black_influence, 1);
+  break_territories(BLACK, &initial_black_influence, 1, NO_MOVE);
 
   set_strength_data(WHITE, safe_stones, strength);
   compute_influence(WHITE, safe_stones, strength, &initial_white_influence,
                     NO_MOVE, "initial white influence, dragons known");
-  break_territories(WHITE, &initial_white_influence, 1);
+  break_territories(WHITE, &initial_white_influence, 1, NO_MOVE);
 }
 
 
@@ -1310,7 +1262,7 @@ compute_dragon_genus(int d, struct eyevalue *genus, int eye_to_exclude)
 	if (eye_to_exclude == NO_MOVE
 	    && (eye_move_urgency(&black_eye[pos].value)
 		> eye_move_urgency(genus)))
-	  DRAGON2(d).heye = black_eye[pos].defense_point;
+	  DRAGON2(d).heye = black_vital_points[pos].defense_points[0];
 
 	add_eyevalues(genus, &black_eye[pos].value, genus);
       }
@@ -1333,7 +1285,7 @@ compute_dragon_genus(int d, struct eyevalue *genus, int eye_to_exclude)
 	if (eye_to_exclude == NO_MOVE
 	    && (eye_move_urgency(&white_eye[pos].value)
 		> eye_move_urgency(genus)))
-	  DRAGON2(d).heye = white_eye[pos].defense_point;
+	  DRAGON2(d).heye = white_vital_points[pos].defense_points[0];
 
 	add_eyevalues(genus, &white_eye[pos].value, genus);
       }
@@ -2410,8 +2362,6 @@ dragon_weak(int pos)
 }
 
 
-
-
 /* Returns the size of the biggest critical dragon on the board. */
 
 int  
