@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -69,6 +69,19 @@ openregion(int i1, int i2, int j1, int j2)
   if (j1 > j2)
     return openregion(i1, i2, j2, j1);
 
+  /* Disregard parts of the region off the board. This is convenient
+   * in order not to have to special-case tiny boards. It also secures
+   * against potential reading outside the board[] array boundaries.
+   */
+  if (i1 < 0)
+    i1 = 0;
+  if (j1 < 0)
+    j1 = 0;
+  if (i2 >= board_size)
+    i2 = board_size - 1;
+  if (j2 >= board_size)
+    j2 = board_size - 1;
+    
   for (x = i1; x <= i2; x++)
     for (y = j1; y <= j2; y++)
       if (BOARD(x, y) != EMPTY)
@@ -207,7 +220,7 @@ choose_corner_move(int corner, int *m, int *n)
 
 /* Announce move, but check for politeness first. */
 static void
-announce_move(int move, int val, int color)
+announce_move(int move, int value, int color)
 {
   int i, j;
   /* This shouldn't happen. */
@@ -238,8 +251,8 @@ announce_move(int move, int val, int color)
       move = POS(board_size - 1 - j, board_size - 1 - i);
   }
   
-  if (set_minimum_move_value(move, val))
-    TRACE("Fuseki Player suggests %1m with value %d\n", move, val);
+  if (set_minimum_move_value(move, value))
+    TRACE("Fuseki Player suggests %1m with value %d\n", move, value);
 }
 
 
@@ -299,12 +312,15 @@ search_fuseki_database(int color)
   if (num_fuseki_moves == 0)
     return 0;
 
-  /* Choose randomly with respect to relative weights for matched moves. */
-  /* Do not choose moves with less value than 20% of the best move */
+  /* Choose randomly with respect to relative weights for matched moves.
+   * Do not choose moves with less value than 20% of the best move
+   * if there are more than two moves on board.
+   */
   best_fuseki_value = fuseki_value[0];
   q = gg_rand() % fuseki_total_value;
   for (k = 0; k < num_fuseki_moves; k++) {
-    if (fuseki_value[k] < (best_fuseki_value / 5))
+    if (stones_on_board(BLACK | WHITE) > 2
+	&& fuseki_value[k] < (best_fuseki_value / 5))
       break;
     q -= fuseki_value[k];
     if (q < 0)
