@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -37,8 +37,8 @@
 #define NEEDS_UPDATE(x) (x != position_number ? (x = position_number, 1) : 0)
 
 static int get_level(int *level);
-static int do_genmove(int *move, int color, float pure_threat_value,
-		      int allowed_moves[BOARDMAX]);
+static int do_genmove(int color, float pure_threat_value,
+		      int allowed_moves[BOARDMAX], float *value, int *resign);
 
 /* Position numbers for which various examinations were last made. */
 static int worms_examined = -1;
@@ -54,9 +54,8 @@ static int revise_thrashing_dragon(int color, float our_score,
 
 static void break_mirror_go(int color);
 static int find_mirror_move(int *move, int color);
-static int should_resign(int color, float our_score);
-
-void sgfShowConsideredMoves(void);
+static int should_resign(int color, float optimistic_score, int move);
+static void compute_scores(void);
 
 
 /* Reset some things in the engine. 
@@ -150,6 +149,7 @@ examine_position(int how_much)
     
     if (NEEDS_UPDATE(dragons_examined)) {
       make_dragons(0);
+      compute_scores();
       /* We have automatically done a partial dragon analysis as well. */
       dragons_examined_without_owl = position_number;
     }
@@ -163,6 +163,7 @@ examine_position(int how_much)
 	   || how_much == EXAMINE_DRAGONS
 	   || how_much == EXAMINE_ALL) {
     initialize_dragon_data();
+    compute_scores();
     verbose = save_verbose;
     gg_assert(test_gray_border() < 0);
     return;
@@ -220,33 +221,31 @@ silent_examine_position(int how_much)
 
 
 /* 
- * Generate computer move for COLOR.
+ * Generate computer move for color.
  *
- * Return the generated move in (*i, *j).
+ * Return the generated move.
  */
 
 int
-genmove(int *i, int *j, int color)
+genmove(int color, float *value, int *resign)
 {
-  int move;
-  int retval;
+  int move = PASS_MOVE;
+  if (resign)
+    *resign = 0;
 
 #if ORACLE
   if (metamachine) {
-    retval = metamachine_genmove(i, j, color);
+    move = metamachine_genmove(color, value);
     gg_assert(stackp == 0);
-    if (*i != -1)
-      return 1;
+    if (move != PASS_MOVE)
+      return move;
   }
 #endif
 
-  retval = do_genmove(&move, color, 0.4, NULL);
-  gg_assert(retval < 0 || ON_BOARD1(move));
+  move = do_genmove(color, 0.4, NULL, value, resign);
+  gg_assert(move == PASS_MOVE || ON_BOARD(move));
 
-  if (i) *i = I(move);
-  if (j) *j = J(move);
-
-  return retval;
+  return move;
 }
 
 
@@ -256,32 +255,16 @@ genmove(int *i, int *j, int color)
  */
 
 int
-genmove_conservative(int *i, int *j, int color)
+genmove_conservative(int color, float *value)
 {
-  int move;
-  int retval;
-
-  retval = do_genmove(&move, color, 0.0, NULL);
-
-  if (i) *i = I(move);
-  if (j) *j = J(move);
-
-  return retval;
+  return do_genmove(color, 0.0, NULL, value, NULL);
 }
 
 
 int
-genmove_restricted(int *i, int *j, int color, int allowed_moves[BOARDMAX])
+genmove_restricted(int color, int allowed_moves[BOARDMAX])
 {
-  int move;
-  int retval;
-
-  retval = do_genmove(&move, color, 0.0, allowed_moves);
-
-  if (i) *i = I(move);
-  if (j) *j = J(move);
-
-  return retval;
+  return do_genmove(color, 0.0, allowed_moves, NULL, NULL);
 }
 
 /* This function collects move reasons can be generated immediately from
@@ -309,24 +292,32 @@ collect_move_reasons(int color)
  */
   
 static int
-do_genmove(int *move, int color, float pure_threat_value,
-	   int allowed_moves[BOARDMAX])
+do_genmove(int color, float pure_threat_value,
+	   int allowed_moves[BOARDMAX], float *value, int *resign)
 {
-  float val;
-  float upper_bound, lower_bound;
-  float average_score, our_score;
+  float average_score, pessimistic_score, optimistic_score;
   int save_verbose;
   int save_depth;
+  int move;
+  float dummy_value;
+  int use_thrashing_dragon_heuristics = 0;
+
+  if (!value)
+    value = &dummy_value;
 
   start_timer(0);
   clearstats();
+
+  /* Usually we would not recommend resignation. */
+  if (resign)
+    *resign = 0;
   
   /* Prepare our table of moves considered. */
   memset(potential_moves, 0, sizeof(potential_moves));
   
   /* no move is found yet. */
-  *move = NO_MOVE;  
-  val = -1; 
+  move = PASS_MOVE;  
+  *value = 0.0; 
   if (get_level(&level))
     fprintf(stderr, "level = %d\n", level);
   
@@ -345,9 +336,10 @@ do_genmove(int *move, int color, float pure_threat_value,
   if (play_mirror_go
       && (mirror_stones_limit < 0
 	  || stones_on_board(WHITE | BLACK) <= mirror_stones_limit)
-      && find_mirror_move(move, color)) {
-    TRACE("genmove() recommends mirror move at %1m\n", *move);
-    return 1.0;
+      && find_mirror_move(&move, color)) {
+    TRACE("genmove() recommends mirror move at %1m\n", move);
+    *value = 1.0;
+    return move;
   }
 
   /* Find out information about the worms and dragons. */
@@ -355,27 +347,6 @@ do_genmove(int *move, int color, float pure_threat_value,
   examine_position(EXAMINE_ALL);
   time_report(1, "examine position", NO_MOVE, 1.0);
 
-  /* Make a score estimate. This can be used in later stages of the 
-   * move generation.  If we are ahead, we can play safely and if
-   * we are behind, we have to play more daringly.
-   */
-  estimate_score(&upper_bound, &lower_bound);
-  if (verbose || showscore) {
-    if (lower_bound == upper_bound)
-      gprintf("\nScore estimate: %s %f\n",
-	      lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound));
-    else
-      gprintf("\nScore estimate: %s %f to %s %f\n",
-	      lower_bound > 0 ? "W " : "B ", gg_abs(lower_bound),
-	      upper_bound > 0 ? "W " : "B ", gg_abs(upper_bound));
-    fflush(stderr);
-  }
-  time_report(1, "estimate score", NO_MOVE, 1.0);
-  if (color == WHITE)
-    average_score = (upper_bound + lower_bound)/2.0;
-  else
-    average_score = -(upper_bound + lower_bound)/2.0;
-  choose_strategy(color, average_score, game_status(color));
 
   /* The score will be used to determine when we are safely
    * ahead. So we want the most conservative score.
@@ -383,17 +354,21 @@ do_genmove(int *move, int color, float pure_threat_value,
    * We always want to have the score from our point of view. So
    * negate it if we are black.
    */
+  if (color == WHITE) {
+    pessimistic_score = black_score;
+    optimistic_score = white_score;
+  }
+  else {
+    pessimistic_score = -white_score;
+    optimistic_score = -black_score;
+  }
+
   if (color == WHITE)
-    our_score = lower_bound;
+    average_score = (white_score + black_score)/2.0;
   else
-    our_score = -upper_bound;
+    average_score = -(white_score + black_score)/2.0;
+  choose_strategy(color, average_score, game_status(color));
 
-  /*
-   * Print some of the information if the user wants to.
-   */
-  if (printmoyo)
-    print_moyo();
-  
   if (printboard) {
     if (printboard == 1)
       fprintf(stderr, "\n          dragon_status display:\n\n");
@@ -428,8 +403,16 @@ do_genmove(int *move, int color, float pure_threat_value,
     fuseki(color);
   gg_assert(stackp == 0);
 
-  /* Look for moves too break mirror play by the opponent. */
+  /* Look for moves to break mirror play by the opponent. */
   break_mirror_go(color);
+
+  /* If we are ahead by 5 points or more, consider a thrashing
+   * dragon dangerous and change its status from DEAD to
+   * UNKNOWN. Otherwise, pretend there is no thrashing dragon.
+   */
+  if (!doing_scoring && !limit_search)
+    use_thrashing_dragon_heuristics
+      = revise_thrashing_dragon(color, pessimistic_score, 5.0);
   
   /* The general pattern database. */
   shapes(color);
@@ -442,38 +425,23 @@ do_genmove(int *move, int color, float pure_threat_value,
   gg_assert(stackp == 0);
 
   /* Review the move reasons and estimate move values. */
-  if (review_move_reasons(move, &val, color, 
-			  pure_threat_value, our_score, allowed_moves))
-    TRACE("Move generation likes %1m with value %f\n", *move, val);
+  if (review_move_reasons(&move, value, color, 
+			  pure_threat_value, pessimistic_score, allowed_moves,
+			  use_thrashing_dragon_heuristics))
+    TRACE("Move generation likes %1m with value %f\n", move, *value);
   gg_assert(stackp == 0);
   time_report(1, "review move reasons", NO_MOVE, 1.0);
 
-  /* If we are ahead by 15 points or more, consider a thrashing
-   * dragon dangerous and change its status from DEAD to
-   * UNKNOWN. This may generate a move.
-   */
-  if (val < 10.0 && !doing_scoring && !limit_search) {
-    if (revise_thrashing_dragon(color, our_score, 15.0)) {
-      shapes(color);
-      if (!disable_endgame_patterns)
-	endgame_shapes(color);
-      if (review_move_reasons(move, &val, color, pure_threat_value, our_score,
-			      allowed_moves)) {
-	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
-	      *move, val); 
-      }
-    }
-    time_report(1, "move reasons with revised thrashing status", NO_MOVE, 1.0);
-  }
 
   /* If the move value is 6 or lower, we look for endgame patterns too. */
-  if (val <= 6.0 && !disable_endgame_patterns && !limit_search) {
+  if (*value <= 6.0 && !disable_endgame_patterns && !limit_search) {
     endgame_shapes(color);
     endgame(color);
     gg_assert(stackp == 0);
-    if (review_move_reasons(move, &val, color, pure_threat_value, our_score,
-			    allowed_moves))
-      TRACE("Move generation likes %1m with value %f\n", *move, val);
+    if (review_move_reasons(&move, value, color, pure_threat_value,
+	  		    pessimistic_score, allowed_moves,
+			    use_thrashing_dragon_heuristics))
+      TRACE("Move generation likes %1m with value %f\n", move, *value);
     gg_assert(stackp == 0);
     time_report(1, "endgame", NO_MOVE, 1.0);
   }
@@ -482,31 +450,31 @@ do_genmove(int *move, int color, float pure_threat_value,
    * status of the opponent group from DEAD to UNKNOWN, then 
    * run shapes and endgame_shapes again. This may turn up a move.
    */
-  if (val < 0.0) {
-    if (revise_thrashing_dragon(color, our_score, 0.0)
-	|| revise_semeai(color)) {
+  if (move == PASS_MOVE) {
+    if (revise_semeai(color)) {
       shapes(color);
       endgame_shapes(color);
-      if (review_move_reasons(move, &val, color, pure_threat_value, our_score,
-			      allowed_moves)) {
+      if (review_move_reasons(&move, value, color, pure_threat_value,
+			      pessimistic_score, allowed_moves,
+			      use_thrashing_dragon_heuristics)) {
 	TRACE("Upon reconsideration move generation likes %1m with value %f\n",
-	      *move, val); 
+	      move, *value); 
       }
     }
-    time_report(1, "move reasons with revised semeai or thrashing status",
+    time_report(1, "move reasons with revised semeai status",
 		NO_MOVE, 1.0);
   }
 
   /* If still no move, fill a remaining liberty. This should pick up
    * all missing dame points.
    */
-  if (val < 0.0 
-      && fill_liberty(move, color)
-      && (!allowed_moves || allowed_moves[*move])) {
-    val = 1.0;
-    TRACE("Filling a liberty at %1m\n", *move);
-    record_top_move(*move, val);
-    move_considered(*move, val);
+  if (move == PASS_MOVE
+      && fill_liberty(&move, color)
+      && (!allowed_moves || allowed_moves[move])) {
+    *value = 1.0;
+    TRACE("Filling a liberty at %1m\n", move);
+    record_top_move(move, *value);
+    move_considered(move, *value);
     time_report(1, "fill liberty", NO_MOVE, 1.0);
   }
 
@@ -514,51 +482,42 @@ do_genmove(int *move, int color, float pure_threat_value,
    * opponent stones, or if the opponent is trying to live inside
    * our territory and we are clearly ahead, generate an aftermath move.
    */
+  if (move == PASS_MOVE && !doing_scoring) {
+    if (play_out_aftermath 
+	|| capture_all_dead 
+	|| (thrashing_dragon && pessimistic_score > 15.0))
+      move = aftermath_genmove(color, 0, allowed_moves);
+      
+    /* If we're instructed to capture all dead opponent stones, generate
+     * a capturing move.
+     */
+    if (move == PASS_MOVE && capture_all_dead) 
+      move = aftermath_genmove(color, 1, allowed_moves);
 
-  if (val < 0.0
-      && !doing_scoring
-      && (play_out_aftermath 
-	  || capture_all_dead 
-	  || (thrashing_dragon && our_score > 15.0))
-      && aftermath_genmove(move, color, NULL, 0) > 0
-      && (!allowed_moves || allowed_moves[*move])) {
-    ASSERT1(is_legal(*move, color), *move);
-    val = 1.0;
-    TRACE("Aftermath move at %1m\n", *move);
-    record_top_move(*move, val);
-    move_considered(*move, val);
-    time_report(1, "aftermath_genmove", NO_MOVE, 1.0);
-  }
-
-  /* If we're instructed to capture all dead opponent stones, generate
-   * a capturing move.
-   */
-  if (val < 0.0
-      && !doing_scoring
-      && capture_all_dead 
-      && aftermath_genmove(move, color, NULL, 1) > 0
-      && (!allowed_moves || allowed_moves[*move])) {
-    ASSERT1(is_legal(*move, color), *move);
-    val = 1.0;
-    TRACE("Aftermath move at %1m\n", *move);
-    move_considered(*move, val);
-    time_report(1, "aftermath_genmove", NO_MOVE, 1.0);
+    if (move != PASS_MOVE) {
+      ASSERT1(is_legal(move, color), move);
+      *value = 1.0;
+      TRACE("Aftermath move at %1m\n", move);
+      record_top_move(move, *value);
+      move_considered(move, *value);
+      time_report(1, "aftermath_genmove", NO_MOVE, 1.0);
+    }
   }
 
   /* If no move is found then pass. */
-  if (val < 0.0) {
+  if (move == PASS_MOVE) {
     TRACE("I pass.\n");
-    *move = NO_MOVE;
   }
   else {
-    TRACE("genmove() recommends %1m with value %f\n", *move, val);
-    /* Maybe time to resign...
-     */
-    if (resign_allowed && val < 10.0 && should_resign(color, our_score)) {
-      TRACE("... though, genmove() thinks the position is hopeless\n");
-      /* Signal resignation by negating the move value */
-      val = -val;
-    }
+    TRACE("genmove() recommends %1m with value %f\n", move, *value);
+  }
+  
+  /* Maybe time to resign...
+   */
+  if (resign && resign_allowed
+      && *value < 10.0 && should_resign(color, optimistic_score, move)) {
+    TRACE("... though, genmove() thinks the position is hopeless\n");
+    *resign = 1;
   }
   
   /* If statistics is turned on, this is the place to show it. */
@@ -566,11 +525,11 @@ do_genmove(int *move, int color, float pure_threat_value,
     showstats();
 
   if (showtime) {
-    double spent = time_report(0, "TIME to generate move at ", *move, 1.0);
+    double spent = time_report(0, "TIME to generate move at ", move, 1.0);
     total_time += spent;
     if (spent > slowest_time) {
       slowest_time = spent;
-      slowest_move = *move;
+      slowest_move = move;
       slowest_movenum = movenum + 1;
     }
   }
@@ -582,7 +541,7 @@ do_genmove(int *move, int color, float pure_threat_value,
   gg_assert(test_gray_border() < 0);
   gg_assert(depth == save_depth);
 
-  return val;
+  return move;
 }
 
 
@@ -593,13 +552,10 @@ do_genmove(int *move, int color, float pure_threat_value,
  */
 
 void 
-move_considered(int move, float val)
+move_considered(int move, float value)
 {
-  int i = I(move);
-  int j = J(move);
-  if (val > potential_moves[i][j]) {
-    potential_moves[i][j] = val;
-  }
+  if (value > potential_moves[move])
+    potential_moves[move] = value;
 }
 
 
@@ -738,6 +694,37 @@ find_mirror_move(int *move, int color)
   return 0;
 }
 
+/* Computer two territory estimates: for *upper, the status of all
+ * cricital stones gets resolved in White's favor; vice verso for
+ * black.
+ */
+static void
+compute_scores()
+{
+  char safe_stones[BOARDMAX];
+  float strength[BOARDMAX];
+
+  set_strength_data(WHITE, safe_stones, strength);
+  compute_influence(EMPTY, safe_stones, strength, &move_influence,
+      		    NO_MOVE, "White territory estimate");
+  white_score = influence_score(&move_influence);
+  set_strength_data(BLACK, safe_stones, strength);
+  compute_influence(EMPTY, safe_stones, strength, &move_influence,
+      		    NO_MOVE, "White territory estimate");
+  black_score = influence_score(&move_influence);
+
+  if (verbose || showscore) {
+    if (white_score == black_score)
+      gprintf("Score estimate: %s %f\n",
+	      black_score > 0 ? "W " : "B ", gg_abs(black_score));
+    else
+      gprintf("Score estimate: %s %f to %s %f\n",
+	      black_score > 0 ? "W " : "B ", gg_abs(black_score),
+	      white_score > 0 ? "W " : "B ", gg_abs(white_score));
+    fflush(stderr);
+  }
+}
+
 
 /* Detect if a white opponent has played mirror go for at least 10
  * moves and if so play on tengen.
@@ -762,16 +749,24 @@ break_mirror_go(int color)
 /* Helper to decide whether GG should resign a game
  */
 static int
-should_resign(int color, float our_score)
+should_resign(int color, float optimistic_score, int move)
 {
   float status;
   int d;
   /* We resign 19x19 games only, smaller board games are fast enough.
    * We resign only if the margin is bigger than 45 pts and if we are
    * behind (of course).
+   *
+   * As an exception to this rule, we resign on any board size if
+   * it looks like all our dragons are dead and the generated move
+   * is a pass.
    */
-  if (board_size < 19
-      || our_score > -45.0) 
+  if (board_size > 2 && move == PASS_MOVE && !lively_dragon_exists(color))
+    return 1;
+  
+  if (move == PASS_MOVE
+      || board_size < 19
+      || optimistic_score > -45.0)
     return 0;
   /* Check dragon statuses. If a friendly dragon is critical, we are
    * possibly not that much behind after we save it. If some hostile
