@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -43,6 +43,7 @@
 
 #include "sgftree.h"
 #include "clock.h"
+#include "winsocket.h"
 
 /* interface.c */
 /* Initialize the whole thing. Should be called once. */
@@ -78,16 +79,6 @@ const char *result_to_string(int result);
 #define SURROUNDED 1
 #define WEAKLY_SURROUNDED 2
 
-/* =================================================================== */
-/*  Configure options that have been removed from the stable version.  */
-/* =================================================================== */
-
-#define EXPERIMENTAL_CONNECTIONS 1
-#define ALTERNATE_CONNECTIONS 1
-#define EXPERIMENTAL_OWL_EXT 0
-#define EXPERIMENTAL_READING 0
-#define OWL_THREATS 0
-
 /* ================================================================ */
 /*                        Board manipulation                        */
 /* ================================================================ */
@@ -109,7 +100,7 @@ int  gnugo_placehand(int handicap);
 int  gnugo_sethand(int handicap, SGFNode *root);
 void gnugo_recordboard(SGFNode *node);
 
-int  gnugo_genmove(int *i, int *j, int color);
+float gnugo_genmove(int *i, int *j, int color, int *resign);
 
 int  gnugo_attack(int m, int n, int *i, int *j);
 int  gnugo_find_defense(int m, int n, int *i, int *j);
@@ -198,6 +189,7 @@ extern int output_flags;       /* amount of output to outfile */
 #define DEBUG_MISCELLANEOUS         0x800000
 #define DEBUG_ORACLE_STREAM         0x1000000
 #define DEBUG_LARGE_SCALE           0x1000000
+#define DEBUG_SPLIT_OWL             0x2000000
 
 /* hash flag bits 
  *
@@ -267,8 +259,7 @@ extern int mandated_owl_branch_depth;
 extern int mandated_owl_reading_depth;
 extern int mandated_owl_node_limit; 
 
-/* Keep this as 2D until we change the entire API. */
-extern float potential_moves[MAX_BOARD][MAX_BOARD];
+extern float potential_moves[BOARDMAX];
 
 extern int limit_search;  /* limit move search to a portion of the board */
 extern int oracle_exists; /* oracle is available for consultation        */
@@ -360,8 +351,8 @@ void change_dragon_status(int dr, int status);
 void who_wins(int color, FILE *outfile);
 
 /* high-level routine to generate the best move for the given color */
-int genmove(int *i, int *j, int color);
-int genmove_conservative(int *i, int *j, int color);
+int genmove(int color, float *value, int *resign);
+int genmove_conservative(int color, float *value);
 
 /* Play through the aftermath. */
 float aftermath_compute_score(int color, float komi, SGFTree *tree);
@@ -379,15 +370,12 @@ int crude_status(int pos);
 int dragon_status(int pos);
 int same_dragon(int dr1, int dr2);
 
-/* moyo functions */
-void print_moyo(void);
-
 /* debugging functions */
 void prepare_pattern_profiling(void);
 void report_pattern_profiling(void);
 
 /* sgffile.c */
-void sgffile_add_debuginfo(SGFNode *node, int value);
+void sgffile_add_debuginfo(SGFNode *node, float value);
 void sgffile_output(SGFTree *tree);
 
 void sgffile_printsgf(int color_to_play, const char *filename);
