@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -35,8 +35,8 @@ static void add_influence_source(int pos, int color, float strength,
                                  float attenuation,
                                  struct influence_data *q);
 static void segment_influence(struct influence_data *q);
-void print_influence(const struct influence_data *q,
-		     const char *info_string);
+static void print_influence(const struct influence_data *q,
+			    const char *info_string);
 static void print_numeric_influence(const struct influence_data *q,
 				    const float values[BOARDMAX],
 				    const char *format, int width,
@@ -68,11 +68,11 @@ static struct influence_data *current_influence = NULL;
 
 
 /* Thresholds values used in the whose_moyo() functions */
-struct moyo_determination_data moyo_data;
-struct moyo_determination_data moyo_restricted_data;
+static struct moyo_determination_data moyo_data;
+static struct moyo_determination_data moyo_restricted_data;
  
 /* Thresholds value used in the whose_territory() function */
-float territory_determination_value; 
+static float territory_determination_value; 
  
 
 
@@ -863,8 +863,8 @@ influence_callback(int anchor, int color, struct pattern *pattern, int ll,
        add_influence_source(pos, this_color, strength, attenuation, q);
 
      DEBUG(DEBUG_INFLUENCE,
-	  "  low intensity influence source at %1m, strength %f, color %C\n",
-	  pos, strength, this_color);
+	   "  low intensity influence source at %1m, strength %f, color %C\n",
+	   pos, strength, this_color);
      return;
   }
     
@@ -1087,6 +1087,98 @@ find_influence_patterns(struct influence_data *q, int color)
   reset_unblocked_blocks(q);
 }
 
+/* This function checks whether we have two or more adjacent blocks for
+ * influence of color next to pos. If yes, it returns the position of the
+ * least valuable blocks; otherwise, it returns NO_MOVE.
+ */
+static int
+check_double_block(int color, int pos, const struct influence_data *q)
+{
+  int k;
+  int block_neighbors = 0;
+  const float *permeability = ((color == BLACK) ? q->black_permeability :
+						  q->white_permeability);
+
+  /* Count neighboring blocks. */
+  for (k = 0; k < 4; k++)
+    if (board[pos + delta[k]] == EMPTY && permeability[pos + delta[k]] == 0.0)
+      block_neighbors++;
+
+  if (block_neighbors >= 2) {
+    /* Search for least valuable block. */
+    float smallest_value = 4.0 * MAX_BOARD * MAX_BOARD;
+    int smallest_block = NO_MOVE;
+    /* We count opponent's territory as positive. */
+    float sign = ((color == WHITE) ? -1.0 : 1.0);
+    for (k = 0; k < 4; k++) {
+      int neighbor = pos + delta[k];
+      if (board[neighbor] == EMPTY && permeability[neighbor] == 0.0) {
+	/* Value is sum of opponents territory at this and all 4 neighboring
+	 * intersections.
+	 */
+	float this_value = sign * q->territory_value[neighbor];
+	int j;
+	for (j = 0; j < 4; j++)
+	  if (ON_BOARD(neighbor + delta[j]))
+	    this_value += sign * q->territory_value[neighbor + delta[j]];
+	/* We use an artifical tie breaker to avoid possible platform
+	 * dependency.
+	 */
+	if (this_value + 0.0005 < smallest_value) {
+	  smallest_block = neighbor;
+	  smallest_value = this_value;
+	}
+      }
+    }
+    ASSERT1(ON_BOARD1(smallest_block), pos);
+    return smallest_block;
+  }
+  return NO_MOVE;
+}
+
+#define MAX_DOUBLE_BLOCKS 20 
+
+
+/* This function checks for the situation where an influence source for
+ * the color to move is direclty neighbored by 2 or more influence blocks.
+ * It then removes the least valuable of these blocks, and re-runs the
+ * influence accumulation for this position.
+ *
+ * See endgame:840 for an example where this is essential.
+ */
+static void
+remove_double_blocks(struct influence_data *q,
+    		     const char inhibited_sources[BOARDMAX])
+{
+  int ii;
+  float *strength = ((q->color_to_move == WHITE) ? q->white_strength :
+      						   q->black_strength);
+  int double_blocks[MAX_DOUBLE_BLOCKS];
+  int num_blocks = 0;
+  for (ii = BOARDMIN; ii < BOARDMAX; ii++)
+    if (board[ii] == EMPTY
+	&& !(inhibited_sources && inhibited_sources[ii])
+	&& strength[ii] > 0.0) {
+      double_blocks[num_blocks] = check_double_block(q->color_to_move, ii, q);
+      if (double_blocks[num_blocks] != NO_MOVE) {
+	num_blocks++;
+	if (num_blocks == MAX_DOUBLE_BLOCKS)
+	  break;
+      }
+    }
+  {
+    int k;
+    float *permeability = ((q->color_to_move == BLACK)
+			   ? q->black_permeability : q->white_permeability);
+    for (k = 0; k < num_blocks; k++) {
+      DEBUG(DEBUG_INFLUENCE, "Removing block for %s at %1m.\n",
+	    q->color_to_move, double_blocks[k]);
+      permeability[double_blocks[k]] = 1.0;
+      accumulate_influence(q, double_blocks[k], q->color_to_move);
+    }
+  }
+}
+
 
 /* Do the real work of influence computation. This is called from
  * compute_influence and compute_escape_influence.
@@ -1115,6 +1207,9 @@ do_compute_influence(int color, const char safe_stones[BOARDMAX],
 	accumulate_influence(q, ii, BLACK);
     }
 
+  value_territory(q);
+  remove_double_blocks(q, inhibited_sources);
+
   value_territory(q);
   segment_influence(q);
   
@@ -1361,6 +1456,20 @@ value_territory(struct influence_data *q)
 	/* If both side have small influence, we have to reduce this value.
 	 * What we consider "small influence" depends on how central this
 	 * intersection lies.
+	 *
+	 * The values of central on an 11x11 board become:
+	 *
+	 *  4  5  6  7  7  7  7  7  6  5  4
+	 *  5  8  9 10 10 10 10 10  9  8  5
+	 *  6  9 12 13 13 13 13 13 12  9  6
+	 *  7 10 13 16 16 16 16 16 13 10  7
+	 *  7 10 13 16 17 17 17 16 13 10  7
+	 *  7 10 13 16 17 18 17 16 13 10  7
+	 *  7 10 13 16 17 17 17 16 13 10  7
+	 *  7 10 13 16 16 16 16 16 13 10  7
+	 *  6  9 12 13 13 13 13 13 12  9  6
+	 *  5  8  9 10 10 10 10 10  9  8  5
+	 *  4  5  6  7  7  7  7  7  6  5  4
 	 */
         dist_i = gg_min(I(ii), board_size - I(ii) - 1);
         dist_j = gg_min(J(ii), board_size - J(ii) - 1);
@@ -1371,6 +1480,14 @@ value_territory(struct influence_data *q)
 	central = (float) 2 * gg_min(dist_i, dist_j) + dist_i + dist_j;
         ratio = gg_max(q->black_influence[ii], q->white_influence[ii])
                 / gg_interpolate(&min_infl_for_territory, central);
+
+	/* Do not make this adjustment when scoring unless both
+	 * players have non-zero influence.
+	 */
+	if (doing_scoring && (q->black_influence[ii] == 0.0
+			      || q->white_influence[ii] == 0.0))
+	  ratio = 1.0;
+	
         first_guess[ii] *= gg_interpolate(&territory_correction, ratio);
 
 	/* Dead stone, upgrade to territory. Notice that this is not
@@ -1863,14 +1980,8 @@ compute_escape_influence(int color, const char safe_stones[BOARDMAX],
 	escape_value[ii] = 0;
     }
 
-  if (0 && (debug & DEBUG_ESCAPE) && verbose > 0) {
-    print_numeric_influence(&escape_influence,
-			    escape_influence.white_influence,
-			    "%3.0f", 3, 1, 1);
-    print_numeric_influence(&escape_influence,
-			    escape_influence.black_influence,
-			    "%3.0f", 3, 1, 1);
-  }    
+  if (0 && (debug & DEBUG_ESCAPE) && verbose > 0)
+    print_influence(&escape_influence, "escape influence");
 
   if (!goal) {
     /* Save the computed values in the cache. */
@@ -1885,7 +1996,7 @@ compute_escape_influence(int color, const char safe_stones[BOARDMAX],
 /* Cache of delta_territory_values. */
 static float delta_territory_cache[BOARDMAX];
 static float followup_territory_cache[BOARDMAX];
-Hash_data delta_territory_cache_hash[BOARDMAX];
+static Hash_data delta_territory_cache_hash[BOARDMAX];
 static int territory_cache_position_number = -1;
 static int territory_cache_influence_id = -1;
 static int territory_cache_color = -1;
@@ -2010,11 +2121,8 @@ influence_delta_territory(const struct influence_data *base,
 
 
 /* Estimate the score. A positive value means white is ahead. The
- * score is estimated from the initial_influence, which must have been
- * computed in advance. The score estimate does not include captured
- * stones (i.e., the ones having been removed from the board) or komi.
- * moyo_coeff and area_coeff are the relative weights to be used for
- * the moyo and area difference respectively.
+ * score is estimated influence data *q, which must have been
+ * computed in advance.
  */
 float
 influence_score(const struct influence_data *q)
@@ -2025,7 +2133,7 @@ influence_score(const struct influence_data *q)
   for (ii = BOARDMIN; ii < BOARDMAX; ii++)
     if (ON_BOARD(ii))
       score += q->territory_value[ii];
-  score += black_captured - white_captured;
+  score += black_captured - white_captured + komi;
 
   return score;
 }
@@ -2211,16 +2319,10 @@ print_influence(const struct influence_data *q, const char *info_string)
   if (printmoyo & PRINTMOYO_NUMERIC_INFLUENCE) {
     /* Print the white influence values. */
     fprintf(stderr, "white influence (%s):\n", info_string);
-    if (q->is_territorial_influence)
-      print_numeric_influence(q, q->white_influence, "%5.1f", 5, 1, 0);
-    else
-      print_numeric_influence(q, q->white_influence, "%3.0f", 3, 1, 1);
+    print_numeric_influence(q, q->white_influence, "%5.1f", 5, 1, 0);
     /* Print the black influence values. */
     fprintf(stderr, "black influence (%s):\n", info_string);
-    if (q->is_territorial_influence)
-      print_numeric_influence(q, q->black_influence, "%5.1f", 5, 1, 0);
-    else
-      print_numeric_influence(q, q->black_influence, "%3.0f", 3, 1, 1);
+    print_numeric_influence(q, q->black_influence, "%5.1f", 5, 1, 0);
   }
 
   if (printmoyo & PRINTMOYO_PRINT_INFLUENCE) {
