@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -286,10 +286,17 @@ gnugo_sethand(int handicap, SGFNode *node)
 
 
 /* Interface to genmove() */
-int
-gnugo_genmove(int *i, int *j, int color)
+float
+gnugo_genmove(int *i, int *j, int color, int *resign)
 {
-  return genmove(i, j, color);
+  float value;
+  int move = genmove(color, &value, resign);
+  if (i && j) {
+    *i = I(move);
+    *j = J(move);
+  }
+   
+  return value;
 }
 
 /* Interface to attack() */
@@ -346,7 +353,11 @@ float
 gnugo_estimate_score(float *upper, float *lower)
 {
   silent_examine_position(EXAMINE_DRAGONS);
-  return estimate_score(upper, lower);
+  if (upper != NULL)
+    *upper = white_score;
+  if (lower != NULL)
+    *lower = black_score;
+  return ((white_score + black_score)/2.0);
 }
 
 
