@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -26,6 +26,7 @@
 #include "board.h"
 #include "hash.h"
 #include "gnugo.h"
+#include "winsocket.h"
 
 /* ================================================================ */
 /*                           public variables                       */
@@ -80,6 +81,7 @@ enum routine_id {
   OWL_CONNECTION_DEFENDS,
   OWL_SUBSTANTIAL,
   OWL_CONFIRM_SAFETY,
+  ANALYZE_SEMEAI,
   NUM_CACHE_ROUTINES
 };
 
@@ -99,7 +101,8 @@ enum routine_id {
   "owl_does_attack", \
   "owl_connection_defends", \
   "owl_substantial", \
-  "owl_confirm_safety"
+  "owl_confirm_safety", \
+  "analyze_semeai"
 
 /* To prioritize between different types of reading, we give a cost
  * ranking to each of the routines above:
@@ -113,7 +116,7 @@ enum routine_id {
  * -1 is left at the end for a consistency check.
  */
 #define ROUTINE_COSTS \
-  3, 3, 4, 0, 0, 1, 1, 2, 2, 3, 3, 3, 3, 3, 3, 3, -1
+  3, 3, 4, 0, 0, 1, 1, 2, 2, 3, 3, 3, 3, 3, 3, 3, 3, -1
   
 
 const char *routine_id_to_string(enum routine_id routine);
@@ -268,10 +271,23 @@ void store_persistent_owl_cache(enum routine_id routine,
 				int tactical_nodes, char goal[BOARDMAX],
 				int goal_color);
 void owl_hotspots(float values[BOARDMAX]);
+int search_persistent_semeai_cache(enum routine_id routine,
+			           int apos, int bpos, int cpos, int color,
+				   Hash_data *goal_hash,
+				   int *resulta, int *resultb,
+				   int *move, int *certain);
+void store_persistent_semeai_cache(enum routine_id routine,
+				   int apos, int bpos, int cpos, int color,
+				   Hash_data *goal_hash,
+				   int resulta, int resultb,
+				   int move, int certain, int tactical_nodes,
+				   char goala[BOARDMAX], char goalb[BOARDMAX]);
+
 
 /* readconnect.c */
 int string_connect(int str1, int str2, int *move);
 int disconnect(int str1, int str2, int *move);
+int fast_disconnect(int str1, int str2, int *move);
 int non_transitivity(int str1, int str2, int str3, int *move);
 
 int break_in(int str, const char goal[BOARDMAX], int *move);
@@ -285,6 +301,7 @@ int vital_chain(int pos);
 int confirm_safety(int move, int color, int *defense_point,
 		   char safe_stones[BOARDMAX]);
 int dragon_weak(int pos);
+float dragon_weakness(int pos, int ignore_dead_dragons);
 int size_of_biggest_critical_dragon(void);
 float blunder_size(int move, int color, int *defense_point,
 		   char safe_stones[BOARDMAX]);
@@ -355,6 +372,8 @@ void add_connection_move(int pos, int dr1, int dr2);
 void add_cut_move(int pos, int dr1, int dr2);
 void add_antisuji_move(int pos);
 void add_semeai_move(int pos, int dr);
+void add_potential_semeai_attack(int pos, int dr1, int dr2);
+void add_potential_semeai_defense(int pos, int dr1, int dr2);
 void add_semeai_threat(int pos, int dr);
 
 void add_owl_attack_move(int pos, int dr, int kworm, int code);
@@ -394,7 +413,7 @@ void add_shape_value(int pos, float value);
 void add_followup_value(int pos, float value);
 void add_reverse_followup_value(int pos, float value);
 int list_move_reasons(FILE *out, int pos);
-void print_all_move_values(void);
+void print_all_move_values(FILE *output);
 void record_top_move(int move, float val);
 void remove_top_move(int move);
 void scale_randomness(int pos, float scaling);
@@ -465,14 +484,14 @@ int atari_atari_confirm_safety(int color, int tpos, int *move, int minsize,
 int atari_atari_blunder_size(int color, int tpos, char defense_moves[BOARDMAX],
 			     const char safe_stones[BOARDMAX]);
 
-int review_move_reasons(int *move, float *val, int color,
+int review_move_reasons(int *move, float *value, int color,
 			float pure_threat_value, float our_score,
-			int allowed_moves[BOARDMAX]);
+			int allowed_moves[BOARDMAX],
+			int use_thrashing_dragon_heuristics);
 void prepare_move_influence_debugging(int pos, int color);
 int fill_liberty(int *move, int color);
-int aftermath_genmove(int *aftermath_move, int color,
-		      int under_control[BOARDMAX],
-		      int do_capture_dead_stones);
+int aftermath_genmove(int color, int do_capture_dead_stones,
+		      int allowed_moves[BOARDMAX]);
 enum dragon_status aftermath_final_status(int color, int pos);
 
 int owl_attack(int target, int *attack_point, int *certain, int *kworm);
@@ -502,10 +521,10 @@ void summon_oracle(void);
 void oracle_loadsgf(char *infilename, char *untilstring);
 int oracle_threatens(int move, int target);
 int within_search_area(int pos);
-int metamachine_genmove(int *i, int *j, int color);
+int metamachine_genmove(int color, float *value);
 void draw_search_area(void);
 
-int genmove_restricted(int *i, int *j, int color, int allowed_moves[BOARDMAX]);
+int genmove_restricted(int color, int allowed_moves[BOARDMAX]);
 
 void change_attack(int str, int move, int acode);
 void change_defense(int str, int move, int dcode);
@@ -523,6 +542,7 @@ int does_attack(int move, int str);
 int does_defend(int move, int str);
 int double_atari(int move, int color, float *value,
 		 char safe_stones[BOARDMAX]);
+int send_two_return_one(int move, int color);
 int play_attack_defend_n(int color, int do_attack, int num_moves, ...);
 int play_attack_defend2_n(int color, int do_attack, int num_moves, ...);
 int play_break_through_n(int color, int num_moves, ...);
@@ -645,11 +665,10 @@ int influence_considered_lively(const struct influence_data *q, int pos);
 void influence_erase_territory(struct influence_data *q, int pos, int color);
 
 void break_territories(int color_to_move, struct influence_data *q,
-			 int store);
+		       int store, int pos);
 void clear_break_in_list(void);
 void break_in_move_reasons(int color);
 
-float estimate_score(float *upper, float *lower);
 void choose_strategy(int color, float our_score, float game_status);
 
 /* Eye space functions. */
@@ -924,12 +943,13 @@ extern struct dragon_data2 *dragon2;
 #if 1 /* Trust DRAGON2 accesses */
 #define DRAGON2(pos) dragon2[dragon[pos].id]
 #else
-struct dragon_data2 * dragon2_func(int pos);
+struct dragon_data2 *dragon2_func(int pos);
 #define DRAGON2(pos) (*dragon2_func(pos))
 #endif
 
 #define DRAGON(d) dragon[dragon2[d].origin]
 
+extern float white_score, black_score;
 
 /* Global variables to tune strategy. */
 
@@ -960,22 +980,19 @@ struct aftermath_data {
 #define MAX_EYE_ATTACKS 3
 
 struct eye_data {
-  int color;/* BLACK, WHITE, BLACK_BORDERED, WHITE_BORDERED or GRAY_BORDERED */
-  int esize;         /* size of the eyespace                                 */
-  int msize;         /* number of marginal vertices                          */
-  int origin;        /* The origin                                           */
-  struct eyevalue value; /* Number of eyes.                                  */
-  int attack_point;  /* vital point for attack */
-  int defense_point; /* vital point for defense	*/
+  int color;             /* BLACK, WHITE, or GRAY                     */
+  int esize;             /* size of the eyespace                      */
+  int msize;             /* number of marginal vertices               */
+  int origin;            /* The origin                                */
+  struct eyevalue value; /* Number of eyes.                           */
 
-  /* The above fields are constant on the whole eyespace. */
+  /* The above fields are constant on the whole eyespace.             */
   /* ---------------------------------------------------------------- */
-  /* The below fields are not. */
+  /* The below fields are not.                                        */
 
-  char marginal;             /* This vertex is marginal                    */
-  char type;                 /* Various characteristics of the eyespace    */
-  char neighbors;            /* number of neighbors in eyespace            */
-  char marginal_neighbors;   /* number of marginal neighbors               */
+  char marginal;             /* This vertex is marginal               */
+  char neighbors;            /* number of neighbors in eyespace       */
+  char marginal_neighbors;   /* number of marginal neighbors          */
 };
 
 struct vital_eye_points {
@@ -1012,11 +1029,6 @@ void compute_eyes_pessimistic(int pos, struct eyevalue *value,
 void propagate_eye(int pos, struct eye_data eye[BOARDMAX]);
 int find_eye_dragons(int origin, struct eye_data eye[BOARDMAX], int eye_color,
 		     int dragons[], int max_dragons);
-float topological_eye(int pos, int color,
-		      struct eye_data my_eye[BOARDMAX],
-		      struct half_eye_data heye[BOARDMAX]);
-void add_false_eye(int pos, struct eye_data eye[BOARDMAX], 
-		   struct half_eye_data heye[BOARDMAX]);
 void make_domains(struct eye_data b_eye[BOARDMAX],
                   struct eye_data w_eye[BOARDMAX],
 		  int owl_call);
