@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -152,7 +152,6 @@ static void matchpat_loop(matchpat_callback_fn_ptr callback,
 			  int color, int anchor,
 			  struct pattern_db *pdb, void *callback_data,
 			  char goal[BOARDMAX], int anchor_in_goal);
-void transform2(int i, int j, int *ti, int *tj, int trans);
 
 /* Precomputed tables to allow rapid checks on the piece at
  * the board. This table relies on the fact that color is
@@ -363,7 +362,7 @@ do_matchpat(int anchor, matchpat_callback_fn_ptr callback, int color,
       int end_transformation;
       int ll;   /* Iterate over transformations (rotations or reflections)  */
       int k;    /* Iterate over elements of pattern */
-      int found_goal, found_nongoal;
+      int found_goal;
   
       /* We can check the color of the anchor stone now.
        * Roughly half the patterns are anchored at each
@@ -467,7 +466,6 @@ do_matchpat(int anchor, matchpat_callback_fn_ptr callback, int color,
 
 	/* Now iterate over the elements of the pattern. */
 	found_goal = 0;
-	found_nongoal = 0;
 	for (k = 0; k < pattern->patlen; ++k) { /* match each point */
 	  int pos; /* absolute coords of (transformed) pattern element */
 	  int att = pattern->patn[k].att;  /* what we are looking for */
@@ -483,12 +481,8 @@ do_matchpat(int anchor, matchpat_callback_fn_ptr callback, int color,
 	  if ((board[pos] & and_mask[color-1][att]) != val_mask[color-1][att])
 	    goto match_failed;
 
-	  if (goal != NULL && board[pos] != EMPTY) {
-	    if (goal[pos])
-	      found_goal = 1;
-	    else if (board[pos] == color)
-	      found_nongoal = 1;
-	  }
+	  if (goal != NULL && board[pos] != EMPTY && goal[pos])
+	    found_goal = 1;
 	  
 	  /* Check out the class_X, class_O, class_x, class_o
 	   * attributes - see patterns.db and above.
@@ -501,12 +495,10 @@ do_matchpat(int anchor, matchpat_callback_fn_ptr callback, int color,
 
 
 #if GRID_OPT == 2
-
 	/* Make sure the grid optimisation wouldn't have 
            rejected this pattern */
 	ASSERT2((merged_val & pattern->and_mask[ll])
 		== pattern->val_mask[ll], m, n);
-
 #endif /* we don't trust the grid optimisation */
 
 
@@ -514,14 +506,9 @@ do_matchpat(int anchor, matchpat_callback_fn_ptr callback, int color,
 	++totals[4];
 #endif
 
-
 	/* Make it here ==> We have matched all the elements to the board. */
 	if ((goal != NULL) && !found_goal)
 	  goto match_failed;
-	if ((goal != NULL) && ((pattern->class) & CLASS_C) && !found_nongoal)
-	  goto match_failed;
-	
-
 
 #ifdef PROFILE_MATCHER
 	++totals[5];
@@ -785,6 +772,8 @@ static const int convert[3][4] = {
   {EMPTY, WHITE, BLACK, OUT_BOARD},	/* WHITE */
   {EMPTY, BLACK, WHITE, OUT_BOARD}	/* BLACK */
 };
+#define EXPECTED_COLOR(player_c, position_c)  		\
+		(convert[player_c][position_c])
 
 /* Forward declarations. */
 static void dfa_prepare_for_match(int color);
@@ -918,7 +907,6 @@ scan_for_patterns(dfa_rt_t *pdfa, int l, int *dfa_pos, int *pat_list)
     row++;
   } while (delta != 0); /* while not on error state */
 
-  gg_assert(row < DFA_MAX_ORDER);
   return id;
 }
 
@@ -983,7 +971,6 @@ check_pattern_light(int anchor, matchpat_callback_fn_ptr callback, int color,
 {
   int k;			/* Iterate over elements of pattern */
   int found_goal = 0;
-  int found_nongoal = 0;
   
 #if PROFILE_PATTERNS
   int nodes_before;
@@ -1020,12 +1007,8 @@ check_pattern_light(int anchor, matchpat_callback_fn_ptr callback, int color,
 
     if (!anchor_in_goal) { 
       /* goal check */
-      if (goal != NULL && board[pos] != EMPTY) {
-        if (goal[pos])
-	  found_goal = 1;
-        else if (board[pos] == color)
-	  found_nongoal = 1;
-      }
+      if (goal != NULL && board[pos] != EMPTY && goal[pos])
+	found_goal = 1;
     }
 
     /* class check */
@@ -1039,8 +1022,6 @@ check_pattern_light(int anchor, matchpat_callback_fn_ptr callback, int color,
   if (!anchor_in_goal) { 
     if (goal != NULL && !found_goal)
       goto match_failed;
-    if (goal != NULL && (pattern->class & CLASS_C) && !found_nongoal)
-      goto match_failed;
   }
 
 #ifdef PROFILE_MATCHER
