@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -44,6 +44,11 @@ int conn_worm1[MAX_CONNECTIONS];
 int conn_worm2[MAX_CONNECTIONS];
 int next_connection;
 
+/* Potential semeai moves. */
+int semeai_target1[MAX_POTENTIAL_SEMEAI];
+int semeai_target2[MAX_POTENTIAL_SEMEAI];
+static int next_semeai;
+
 /* Unordered sets (currently pairs) of move reasons / targets */
 Reason_set either_data[MAX_EITHER];
 int next_either;
@@ -89,6 +94,7 @@ clear_move_reasons(void)
   int k;
   next_reason = 0;
   next_connection = 0;
+  next_semeai = 0;
   next_either = 0;
   next_all = 0;
   next_eye = 0;
@@ -588,7 +594,8 @@ static int
 concerns_noncritical_dragon(int pos, int what)
 {
   UNUSED(pos);
-  return dragon[what].status != CRITICAL; 
+  return (dragon[what].status != CRITICAL
+	  && worm[what].attack_codes[0] == 0);
 }
 
 
@@ -855,6 +862,46 @@ add_semeai_move(int pos, int dr)
   add_move_reason(pos, SEMEAI_MOVE, dragon[dr].origin);
 }
 
+/*
+ * Add to the reasons for the move at (pos) that it might
+ * kill/save the dragon at (dr1) in the semeai against (dr2).
+ */
+static void
+add_potential_semeai_move(int pos, int type, int dr1, int dr2)
+{
+  ASSERT1(ON_BOARD(dr1), pos);
+  ASSERT1(ON_BOARD(dr2), pos);
+  if (next_semeai >= MAX_POTENTIAL_SEMEAI)
+    DEBUG(DEBUG_MOVE_REASONS,
+	  "Potential semeai move at %1m dropped as list was full\n", pos);
+  else {
+    semeai_target1[next_semeai] = dr1;
+    semeai_target2[next_semeai] = dr2;
+    add_move_reason(pos, type, next_semeai);
+    next_semeai++;
+  }
+}
+
+/*
+ * Add to the reasons for the move at (pos) that it might
+ * kill the dragon at (dr1) in the semeai against (dr2).
+ */
+void
+add_potential_semeai_attack(int pos, int dr1, int dr2)
+{
+  add_potential_semeai_move(pos, POTENTIAL_SEMEAI_ATTACK, dr1, dr2);
+}
+
+/*
+ * Add to the reasons for the move at (pos) that it might
+ * save the dragon at (dr1) in the semeai against (dr2).
+ */
+void
+add_potential_semeai_defense(int pos, int dr1, int dr2)
+{
+  add_potential_semeai_move(pos, POTENTIAL_SEMEAI_DEFENSE, dr1, dr2);
+}
+
 /*
  * Add to the reasons for the move at (pos) that given two
  * moves in a row a move here can win the dragon (friendly or
