@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -39,7 +39,6 @@
 #define SEMEAI_MOVE             18
 #define SEMEAI_THREAT           (SEMEAI_MOVE | THREAT_BIT)
 
-/*      BLOCK_TERRITORY_MOVE    20   OBSOLETE */
 #define EXPAND_TERRITORY_MOVE   20
 #define EXPAND_MOYO_MOVE        22
 #define INVASION_MOVE           24
@@ -64,6 +63,8 @@
 
 #define OWL_ATTACK_MOVE_GAIN    60
 #define OWL_DEFEND_MOVE_LOSS    62
+#define POTENTIAL_SEMEAI_ATTACK	64
+#define POTENTIAL_SEMEAI_DEFENSE 66
 
 #define ANTISUJI_MOVE           70
 
@@ -136,6 +137,7 @@ struct move_data {
 #define MAX_WORMS		2*MAX_BOARD*MAX_BOARD/3
 #define MAX_DRAGONS		MAX_WORMS
 #define MAX_CONNECTIONS 	4*MAX_WORMS
+#define MAX_POTENTIAL_SEMEAI	50
 #define MAX_EYES		MAX_BOARD*MAX_BOARD/2
 #define MAX_LUNCHES		MAX_WORMS
 #define MAX_EITHER		100
@@ -152,6 +154,9 @@ extern int conn_worm1[MAX_CONNECTIONS];
 extern int conn_worm2[MAX_CONNECTIONS];
 extern int next_connection;
 
+extern int semeai_target1[MAX_POTENTIAL_SEMEAI];
+extern int semeai_target2[MAX_POTENTIAL_SEMEAI];
+
 /* Unordered sets (currently pairs) of move reasons / targets */
 typedef struct {
   int reason1;
