@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -82,8 +82,15 @@ struct oracle_move_data {
   const char *reason; /* why this move */
 };
 
+static void oracle_callback(int anchor, int color, struct pattern *pattern,
+			    int ll, void *data);
+static void oracle_add_move(struct oracle_move_data *moves, 
+			    int this_move, int this_value, 
+			    const char *this_reason);
+void do_consult_oracle(int color);
 void error(const char *msg);
-static int oracle_trymove(int pos, int color, const char *message, int str);
+static int oracle_trymove(int pos, int color, const char *message, int str,
+			  int komaster, int kom_pos);
 static void oracle_popgo(void);
 static void tell_oracle(const char *fmt, ...);
 static void ask_oracle(void);
@@ -162,7 +169,8 @@ error(const char *msg)
  * play the move.
  */
 static int 
-oracle_trymove(int pos, int color, const char *message, int str)
+oracle_trymove(int pos, int color, const char *message, int str,
+	       int komaster, int kom_pos)
 {
   if (!trymove(pos, color, message, str))
     return 0;
@@ -255,6 +263,74 @@ oracle_clear_board(int boardsize)
   ASK_ORACLE;
 }
 
+/*** * *** * *** * *** * *** * *** * *** * *** * *** * *** * *** * ***\
+ *              Demonstration: a pattern matcher                     *
+\*** * *** * *** * *** * *** * *** * *** * *** * *** * *** * *** * ***/
+
+/* Call the pattern matcher */
+
+void
+consult_oracle(int color)
+{
+  do_consult_oracle(color);
+}
+
+void
+do_consult_oracle(int color)
+{
+  struct oracle_move_data oracle_moves[MAX_ORACLE_MOVES];
+  int k;
+
+  for (k = 0; k < MAX_ORACLE_MOVES; k++)
+    oracle_moves[k].value = -1;
+  
+  matchpat(oracle_callback, color, &oracle_db, oracle_moves, NULL);
+  for (k = 0; k < MAX_ORACLE_MOVES; k++)
+    if (oracle_moves[k].value > -1) {
+      oracle_trymove(oracle_moves[k].pos, color, oracle_moves[k].reason,
+		     0, 0, NO_MOVE);
+      do_consult_oracle(OTHER_COLOR(color));
+      oracle_popgo();
+    }
+}
+
+static void
+oracle_callback(int anchor, int color, struct pattern *pattern,
+		int ll, void *data)
+{
+  int this_move;
+  struct oracle_move_data *moves = data;
+  UNUSED(color);
+
+  this_move = AFFINE_TRANSFORM(pattern->move_offset, ll, anchor);
+  if (within_search_area(this_move))
+    oracle_add_move(moves, this_move, pattern->value, pattern->name);
+  else
+    gprintf("outside the area\n");
+}
+
+/* Add a move to a list */
+
+static void
+oracle_add_move(struct oracle_move_data moves[MAX_ORACLE_MOVES],
+		int this_move, int this_value, const char *this_reason)
+{
+  int k, l;
+  
+  for (k = 0; k < MAX_ORACLE_MOVES; k++)
+    if (moves[k].value == -1
+	|| this_value >= moves[k].value)
+      break;
+  for (l = MAX_ORACLE_MOVES-1; l > k; l--) {
+    moves[l].pos = moves[l-1].pos;
+    moves[l].value = moves[l-1].value;
+    moves[l].reason = moves[l-1].reason;
+  }
+  moves[k].pos = this_move;
+  moves[k].value = this_value;
+  moves[k].reason = this_reason;
+}
+
 /*** * *** * *** * *** * *** * *** * *** * *** * *** * *** * *** * ***\
  *              Demonstration: metamachine                           *
 \*** * *** * *** * *** * *** * *** * *** * *** * *** * *** * *** * ***/
@@ -262,13 +338,13 @@ oracle_clear_board(int boardsize)
 #define FIRST_LEVEL_MOVES 3
 #define SECOND_LEVEL_MOVES 2
 
-static float
-do_metamachine_genmove(int *i, int *j, int color, int width);
+static int
+do_metamachine_genmove(int color, int width, float *value);
 
 int
-metamachine_genmove(int *i, int *j, int color)
+metamachine_genmove(int color, float *value)
 {
-  float value;
+  int move;
   int pos;
 
   if (limit_search) {
@@ -284,34 +360,34 @@ metamachine_genmove(int *i, int *j, int color)
       }	
   }
   count_variations = 1;
-  value = do_metamachine_genmove(i, j, color, search_width());
+  move = do_metamachine_genmove(color, search_width(), value);
   sgffile_enddump(outfilename);
   count_variations = 0;
-  return value;
+  return move;
 }
 
-static float
-do_metamachine_genmove(int *i, int *j, int color, int width)
+static int
+do_metamachine_genmove(int color, int width, float *value)
 {
   int k, moves_considered;
   float move_value[10];
   float best_score = 0.;
   int best_move = -1;
   char *token;
-  int move_i[10], move_j[10];
+  int moves[10];
   float score[10];
   char delimiters[] = " \t\r\n";
   char buf[100];
+  int i, j;
 
-  if (color == BLACK) 
+  if (color == BLACK)
     TELL_ORACLE("top_moves_black\n");
   else
     TELL_ORACLE("top_moves_white\n");
   ask_oracle();
   token = strtok(gnugo_line, delimiters);
   for (k = 0; k < 10; k++) {
-    move_i[k] = -1;
-    move_j[k] = -1;
+    moves[k] = PASS_MOVE;
     move_value[k] = 0.0;
   }
   moves_considered = width;
@@ -321,39 +397,36 @@ do_metamachine_genmove(int *i, int *j, int color, int width)
     token = strtok(NULL, delimiters);
     if (!token)
       break;
-    string_to_location(board_size, token, move_i+k, move_j+k);
+    string_to_location(board_size, token, &i, &j);
+    moves[k] = POS(i, j);
     token = strtok(NULL, delimiters);
     if (!token)
       break;
-    sscanf(token, "%f", move_value+k);
-    TRACE("move %d: %m valued %f\n", k,
-	  move_i[k], move_j[k], move_value[k]);
+    sscanf(token, "%f", move_value + k);
+    TRACE("move %d: %1m valued %f\n", k, moves[k], move_value[k]);
   }
   /* if we left the loop early, k is the number of valid moves */
   moves_considered = k;
   if (moves_considered == 0) {
-    *i = -1;
-    *j = -1;
-    return 0;
+    *value = 0.0;
+    return PASS_MOVE;
   }
   if (moves_considered == 1) {
-    *i = move_i[0];
-    *j = move_j[0];
-    return 1;
+    *value = 1.0;
+    return moves[k];
   }
   for (k = 0; k < moves_considered; k++) {
-    if (oracle_trymove(POS(move_i[k], move_j[k]), color, "", 0)) {
+    if (oracle_trymove(moves[k], color, "", 0, 0, NO_MOVE)) {
       int new_width = search_width();
 
       if (new_width == 0) {
 	TELL_ORACLE("experimental_score %s\n", 
 		    color == BLACK ? "black" : "white");
 	ask_oracle();
-	sscanf(gnugo_line, "= %f", score+k);
+	sscanf(gnugo_line, "= %f", score + k);
       }
       else {
-	score[k] =
-	  do_metamachine_genmove(i, j, OTHER_COLOR(color), new_width);
+	do_metamachine_genmove(OTHER_COLOR(color), new_width, &score[k]);
       }
       if (verbose)
 	dump_stack();
@@ -370,10 +443,9 @@ do_metamachine_genmove(int *i, int *j, int color, int width)
       best_score = score[k];
     }
   }
-  TRACE("best: %f at %2m\n", best_score, move_i[best_move], move_j[best_move]);
-  *i = move_i[best_move];
-  *j = move_j[best_move];
-  return score[best_move];
+  TRACE("best: %f at %1m\n", best_score, moves[best_move]);
+  *value = score[best_move];
+  return moves[best_move];
 }
 
 /* decide how wide to search */
