@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,13 +16,14 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 #include "board.h"
 #include "hash.h"
 #include "gg_utils.h"
+#include "sgftree.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -361,7 +362,7 @@ location_to_buffer(int pos, char *buf)
  */
 
 int
-string_to_location(int boardsize, char *str, int *m, int *n)
+string_to_location(int boardsize, const char *str, int *m, int *n)
 {
   if (*str == '\0')
     return 0;
@@ -509,6 +510,25 @@ simple_showboard(FILE *outfile)
 }
 
 
+/* Adds square marks for each goal intersecion in the current sgf_dumptree.
+ * This function cannot be in sgf/ as it has to understand the 1-D board.
+ */
+void
+mark_goal_in_sgf(char goal[BOARDMAX])
+{
+  int pos;
+  SGFNode *node;
+
+  if (!sgf_dumptree)
+    return;
+  node = sgftreeNodeCheck(sgf_dumptree);
+
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++)
+    if (ON_BOARD(pos) && goal[pos])
+      sgfSquare(node, I(pos), J(pos));
+}
+
+
 
 /*
  * Local Variables:
