@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -638,6 +638,29 @@ decide_surrounded(int pos)
 }  
 
 
+#if ORACLE
+
+void
+decide_oracle(Gameinfo *gameinfo, char *infilename, char *untilstring)
+{
+  SGFTree tree;
+
+  reset_engine();
+  if (*outfilename)
+    sgffile_begindump(&tree);
+
+  count_variations = 1;
+  summon_oracle();
+  oracle_loadsgf(infilename, untilstring);
+  consult_oracle(gameinfo->to_move);
+  sgffile_enddump(outfilename);
+  dismiss_oracle();
+  count_variations = 0;
+}
+
+#endif
+
+
 /*
  * Local Variables:
  * tab-width: 8
