@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,18 +16,17 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
 /*
  * sgffile.c
  *
- * This file contains functions that create an SGF file on the fly.
- * There can be only one file open simultaneously.
- *
- * See sgf/sgftree.c for more general handling of SGF trees and file I/O.
+ * This file used to contain functions that create an SGF file on the fly.
  *
+ * Today it contains supporting code around the more general SGF library
+ * found in the sgf/ directory.
  */
 
 #include "gnugo.h"
@@ -47,39 +46,35 @@
  */
 
 void
-sgffile_add_debuginfo(SGFNode *node, int value)
+sgffile_add_debuginfo(SGFNode *node, float value)
 {
-  int m, n;
+  int pos;
   char comment[24];
 
   if (!outfilename[0])
     return;
   
-  for (m = 0; m < board_size; ++m)
-    for (n = 0; n < board_size; ++n) {
-      if (BOARD(m, n) && (output_flags & OUTPUT_MARKDRAGONS)) {
-	switch (dragon[POS(m, n)].crude_status) {
-	case DEAD:
-	  sgfLabel(node, "X", m, n);
-	  break;
-	case CRITICAL:
-	  sgfLabel(node, "!", m, n);
-	  break;
-	default:
-	  ;
-	}
-      }
-      
-      if (potential_moves[m][n] > 0.0 && (output_flags & OUTPUT_MOVEVALUES)) {
-	if (potential_moves[m][n] < 1.0)
-	  sgfLabel(node, "<1", m, n);
-	else
-	  sgfLabelInt(node, (int) potential_moves[m][n], m, n);
-      }
+  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
+    if (!ON_BOARD(pos))
+      continue;
+    
+    if (IS_STONE(board[pos]) && (output_flags & OUTPUT_MARKDRAGONS)) {
+      if (dragon[pos].crude_status == DEAD)
+	sgfLabel(node, "X", I(pos), J(pos));
+      else if (dragon[pos].crude_status == CRITICAL)
+	sgfLabel(node, "!", I(pos), J(pos));
     }
+	
+    if (potential_moves[pos] > 0.0 && (output_flags & OUTPUT_MOVEVALUES)) {
+      if (potential_moves[pos] < 1.0)
+	sgfLabel(node, "<1", I(pos), J(pos));
+      else
+	sgfLabelInt(node, (int) potential_moves[pos], I(pos), J(pos));
+    }
+  }
   
-  if (value > 0 && (output_flags & OUTPUT_MOVEVALUES)) {
-    sprintf(comment, "Value of move: %d", value);
+  if (value > 0.0 && (output_flags & OUTPUT_MOVEVALUES)) {
+    sprintf(comment, "Value of move: %.2f", value);
     sgfAddComment(node, comment);
   }
 }
