@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -84,9 +84,11 @@ cut_possible(int pos, int color)
 
 
 /*
- * does_attack(move, str) returns true if the move at (move)
- * attacks (str). This means that it captures the string, and that
- * (str) is not already dead.
+ * does_attack(move, str) returns the result code for an attack on the
+ * string 'str' by the move 'move'. However, if the move does not
+ * improve the attack result compared to tenuki, 0 is returned. In
+ * particular if the string is already captured, does_attack() always
+ * returns 0.
  */
 
 int
@@ -99,25 +101,23 @@ does_attack(int move, int str)
   int dcode = 0;
   int spos = NO_MOVE;
   
-  if (stackp == 0) {
-    if (worm[str].attack_codes[0] != 0 
-	&& worm[str].defense_codes[0] == 0)
-      return 0;
-    spos = worm[str].defense_points[0];
-  }
-  else {
-    attack_and_defend(str, &acode, NULL, &dcode, &spos);
-    if (acode != 0 && dcode == 0)
-      return 0;
-  }
+  attack_and_defend(str, &acode, NULL, &dcode, &spos);
+  if (acode != 0 && dcode == 0)
+    return 0;
   
   if (trymove(move, other, "does_attack-A", str)) {
-    if (!board[str] || !find_defense(str, NULL)) {
+    if (!board[str])
       result = WIN;
+    else
+      result = REVERSE_RESULT(find_defense(str, NULL));
+    if (result != 0) {
       increase_depth_values();
       if (spos != NO_MOVE && trymove(spos, color, "does_attack-B", str)) {
-	if (board[str] && !attack(str, NULL))
-	  result = 0;
+	if (board[str]) {
+	  int new_result = attack(str, NULL);
+	  if (new_result < result)
+	    result = new_result;
+	}
 	popgo();
       }
       decrease_depth_values();
@@ -125,6 +125,9 @@ does_attack(int move, int str)
     popgo();
   }
 
+  if (result < acode)
+    result = 0;
+  
   return result;
 }
 
@@ -133,6 +136,8 @@ does_attack(int move, int str)
  * does_defend(move, str) returns true if the move at (move)
  * defends (str). This means that it defends the string, and that
  * (str) can be captured if no defense is made.
+ *
+ * FIXME: Make does_defend() ko aware like does_attack().
  */
 
 int
@@ -143,13 +148,7 @@ does_defend(int move, int str)
   int result = 0;
   int spos = NO_MOVE;
 
-  if (stackp == 0) {
-    if (worm[str].attack_codes[0] == 0)
-      return 0;
-    else
-      spos = worm[str].attack_points[0];
-  }
-  else if (!attack(str, &spos))
+  if (!attack(str, &spos))
     return 0;
 
   gg_assert(spos != NO_MOVE);
@@ -195,10 +194,13 @@ somewhere(int color, int check_alive, int num_moves, ...)
     pos = va_arg(ap, int);
 
     if (board[pos] == color
-	&& (!check_alive || dragon[pos].status != DEAD))
+	&& (!check_alive || dragon[pos].status != DEAD)) {
+      va_end(ap);
       return 1;
+    }
   }
 
+  va_end(ap);
   return 0;
 }
 
@@ -296,8 +298,8 @@ test_symmetry_after_move(int move, int color, int strict)
     sum = board[pos] + board[MIRROR_MOVE(pos)];
     if (sum != EMPTY + EMPTY && sum != BLACK + WHITE) {
       if (strict || sum == EMPTY + WHITE || sum == EMPTY + BLACK) {
-      result = 0;
-      break;
+	result = 0;
+	break;
       }
     }
   }
@@ -686,9 +688,6 @@ set_depth_values(int level, int report_levels)
 
   /*
    * Other policies depending on level:
-   * aftermath.c:   >=  8: call estimate_score().
-   * dragon.c:      >=  8: compute owl threats (currently disabled)
-   * genmove.c:     >=  8: call estimate_score().
    * owl.c:         >=  9: use vital attack pattern database
    *                >=  8: increase depth values in owl_substantial
    *                >=  8: don't turn off owl_phase in semeai reading
@@ -1380,481 +1379,60 @@ double_atari(int move, int color, float *value, char safe_stones[BOARDMAX])
 }
     
 
-static int
-capture_non_invincible_strings(int color, int exceptions[BOARDMAX])
-{
-  int other = OTHER_COLOR(color);
-  int something_captured = 1; /* To get into the first turn of the loop. */
-  int moves_played = 0;
-  int save_moves;
-  int libs[MAXLIBS];
-  int liberties;
-  int pos;
-  int k;
-  
-  while (something_captured) {
-    /* Nothing captured so far in this turn of the loop. */
-    something_captured = 0;
-
-    /* Visit all friendly strings on the board. */
-    for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-      if (board[pos] != color || find_origin(pos) != pos)
-	continue;
-
-      if (exceptions && exceptions[pos])
-	continue;
-      
-      /* Try to capture the string at pos. */
-      liberties = findlib(pos, MAXLIBS, libs);
-      save_moves = moves_played;
-      for (k = 0; k < liberties; k++) {
-	if (trymove(libs[k], other, "unconditional_life", pos))
-	  moves_played++;
-      }
-      
-      /* Successful if already captured or a single liberty remains.
-       * Otherwise we must rewind and take back the last batch of moves.
-       */
-      if (board[pos] == EMPTY)
-	something_captured = 1;
-      else if (findlib(pos, 2, libs) == 1) {
-	/* Need to use tryko as a defense against the extreme case
-	 * when the only opponent liberty that is not suicide is an
-	 * illegal ko capture, like in this 5x5 position:
-	 * +-----+
-	 * |.XO.O|
-	 * |XXOO.|
-	 * |X.XOO|
-	 * |XXOO.|
-	 * |.XO.O|
-	 * +-----+
-	 */
-	int success = tryko(libs[0], other, "unconditional_life");
-	gg_assert(success);
-	moves_played++;
-	something_captured = 1;
-      }
-      else
-	while (moves_played > save_moves) {
-	  popgo();
-	  moves_played--;
-	}
-    }
-  }
-  return moves_played;
-}
-
-/* Find those worms of the given color that can never be captured,
- * even if the opponent is allowed an arbitrary number of consecutive
- * moves. The coordinates of the origins of these worms are written to
- * the worm arrays and the number of non-capturable worms is
- * returned.
+/* Returns true if a move by (color) fits a shape like:
  *
- * The algorithm is to cycle through the worms until none remains or
- * no more can be captured. A worm is removed when it is found to be
- * capturable, by letting the opponent try to play on all its
- * liberties. If the attack fails, the moves are undone. When no more
- * worm can be removed in this way, the remaining ones are
- * unconditionally alive.
+ *  -----
+ *  O.O*X        (O=color)
+ *  OOXXX
  *
- * After this, unconditionally dead opponent worms and unconditional
- * territory are identified. This is almost, but only almost,
- * straightforward. We first present a simple but only almost correct
- * solution, then show how to patch up its deficiencies.
- *
- * - - - - - - -
- *
- * Algorithm 1, simple but slightly incorrect.
- *
- * To find unconditionally dead opponent worms and unconditional
- * territory, we continue from the position obtained at the end of the
- * previous operation (only unconditionally alive strings remain for
- * color) with the following steps:
- *
- * 1. Play opponent stones on all liberties of the unconditionally
- *    alive strings except where illegal. (That the move order may
- *    determine exactly which liberties can be played legally is not
- *    important. Just pick an arbitrary order).
- * 2. Recursively extend opponent strings in atari, except where this
- *    would be suicide.
- * 3. Play an opponent stone anywhere it can get two empty
- *    neighbors. (I.e. split big eyes into small ones).
- * 4. Play an opponent stone anywhere it can get one empty
- *    neighbor. (I.e. reduce two space eyes to one space eyes.)
- *
- * Remaining opponent strings in atari and remaining liberties of the
- * unconditionally alive strings constitute the unconditional
- * territory.
- *
- * Opponent strings from the initial position placed on
- * unconditional territory are unconditionally dead.
- *
- * - - - - - - -
- *
- * The deficiency with this algorithm is that a certain class of sekis
- * are considered as dead, e.g. this position:
- *
- * .OOOOO.
- * OOXXXOO
- * OXX.XXO
- * OX.O.XO
- * OX.O.XO
- * OXX.XXO
- * OOXXXOO
- * .OOOOO.
- *
- * The problem is that while removing the two O stones, X is reduced
- * to a single small eye. Still O cannot capture these stones under
- * alternating play since the eyespace is too big.
- *
- * Before discussing this seki further we make a preliminary
- * modification of the algorithm.
- *
- * - - - - - - -
- *
- * Algorithm 2. More complex but still slightly incorrect algorithm:
- *
- * 1. Run algorithm 1.
- * 2. Return to the original position.
- * 3. Capture all capturable O strings which according to algorithm 1
- *    do not belong to unconditional territory.
- * 4. Play opponent stones on all liberties of the unconditionally
- *    alive strings except where illegal. (That the move order may
- *    determine exactly which liberties can be played legally is not
- *    important. Just pick an arbitrary order).
- * 5. Recursively extend opponent strings in atari, except where this
- *    would be suicide.
- * 6. Capture all remaining capturable O strings.
- * 7. Repeat 4 and 5 once.
- * 8. Play an opponent stone anywhere it can get two empty
- *    neighbors. (I.e. split big eyes into small ones).
- * 9. Play an opponent stone anywhere it can get one empty
- *    neighbor. (I.e. reduce two space eyes to one space eyes.)
- *
- * Remaining opponent strings in atari and remaining liberties of the
- * unconditionally alive strings constitute the unconditional
- * territory.
- *
- * Opponent strings from the initial position placed on
- * unconditional territory are unconditionally dead.
- *
- * - - - - - - -
- *
- * We can observe that, after step 5, an X group with at least two
- * distinct eyespaces would not risk being reduced to a single small
- * eye. Similarly an X group with a capturable O string of size at
- * least three would allow the formation of two distinct small eyes
- * after being captured. Thus it is easy to see that the only X groups
- * which would live in seki but could not be transformed into
- * unconditionally alive groups would have a single eyespace with a
- * capturable O string of size at most 2. Furthermore the eyespace
- * would not be possible to subdivide. Then if the capturable string
- * would be of size 1 it would in all cases form a nakade and we would
- * not have a seki. The plausible seki positions would all be
- * reducable to the following eyeshape:
- *
- * .OOOOO.
- * OOXXXO.
- * OXX.XOO
- * OX.OXXO
- * OXXO.XO
- * OOX.XXO
- * .OXXXOO
- * .OOOOO.
- *
- * The remaining question is what effects cutting points in the X
- * group would have. For example these X groups are dead:
- *
- * .OOOOO.   .OOOOO.   .OOOOO.   .OOOOO.   ..OOOO.   ..OOOO.
- * .OXXXO.   .OXXXO.   .OXXXO.	 .OXXXO.   OOOXXO.   OOOXXO.
- * OOX.XO.   OOX.XOO   OOX.XOO	 OOX.XOO   OXX.XO.   OXX.XOO
- * OX.OXOO   OX.OXXO   OX.OXXO	 OX.OXXO   OX.OXOO   OX.OXXO
- * OXXO.XO   OXXO.XO   OXXO.XO	 OXXO.XO   OXXO.XO   OXXO.XO
- * OOX.XXO   OOX.XOO   OOX.XXO	 OOX.XXO   OOX.XXO   OOX.XXO
- * .OXXXOO   .OXXXO.   .OXXOOO	 .OOXXOO   .OXXXOO   .OXXOOO
- * .OOOOO.   .OOOOO.   .OOOO..	 ..OOOO.   .OOOOO.   .OOOO..
- *
- * while these are alive in seki
- *
- * ..OOOO.   .OOOO..   .OOOO..   ..OOOO.   ..OOOO.
- * OOOXXO.   .OXXOO.   OOXXOO.	 .OOXXO.   OOOXXO.
- * OXX.XOO   OOX.XOO   OXX.XOO	 OOX.XOO   OXX.XOO
- * OX.OXXO   OX.OXXO   OX.OXXO	 OX.OXXO   OX.OXXO
- * OXXO.XO   OXXO.XO   OOXO.XO	 OXXO.XO   OOXO.XO
- * OOX.XXO   OOX.XXO   .OX.XXO	 OOX.XXO   .OX.XXO
- * .OXXXOO   .OXXXOO   .OXXOOO	 .OXXXOO   .OXXXOO
- * .OOOOO.   .OOOOO.   .OOOO..	 ..OOOO.   .OOOOO.
- *
- * The critical distinction between the dead ones and the seki ones is
- * that the stones marked a and b below,
- *
- * .OOOOO.
- * OOXXXO.
- * OXX.XOO
- * OX.ObXO
- * OXaO.XO
- * OOX.XXO
- * .OXXXOO
- * .OOOOO.
- *
- * belong to different strings for the dead groups and to the same
- * string for the seki groups.
- *
- * The trick to avoid misclassifying areas where the opponent can form
- * a seki group but not an invincible group as unconditional territory
- * is thus to detect the formation above and add a third stone to the
- * O group before the capturing in step 6 above.
- *
- * This leads to the final algorithm.
- *
- * - - - - - - -
- *
- * Algorithm 3. Final and correct algorithm:
- *
- * 1. Run algorithm 1.
- * 2. Return to the original position.
- * 3. Capture all capturable O strings which according to algorithm 1
- *    do not belong to unconditional territory.
- * 4. Play opponent stones on all liberties of the unconditionally
- *    alive strings except where illegal. (That the move order may
- *    determine exactly which liberties can be played legally is not
- *    important. Just pick an arbitrary order).
- * 5. Recursively extend opponent strings in atari, except where this
- *    would be suicide.
- * 6. Identify eyespaces of the kind described above and extend any
- *    matching two-stone string with a third stone.
- * 7. Capture all remaining capturable O strings.
- * 8. Repeat 4 and 5 once.
- * 9. Play an opponent stone anywhere it can get two empty
- *    neighbors. (I.e. split big eyes into small ones).
- * 10. Play an opponent stone anywhere it can get one empty
- *    neighbor. (I.e. reduce two space eyes to one space eyes.)
- *
- * Remaining opponent strings in atari and remaining liberties of the
- * unconditionally alive strings constitute the unconditional
- * territory.
- *
- * Opponent strings from the initial position placed on
- * unconditional territory are unconditionally dead.
- *
- * - - - - - - -
- *
- * On return, unconditional_territory[][] is 1 where color has
- * unconditionally alive stones, 2 where it has unconditional
- * territory, and 0 otherwise.
+ * More specifically the move should have the following properties:
+ * - The move is a self-atari
+ * - The move forms a string of exactly two stones
+ * - When the opponent captures, the capturing stone becomes a single
+ *   stone in atari
+ * - When capturing back the original position is repeated
  */
 
-void
-unconditional_life(int unconditional_territory[BOARDMAX], int color)
+int
+send_two_return_one(int move, int color)
 {
-  int found_one;
   int other = OTHER_COLOR(color);
-  int libs[MAXLIBS];
-  int liberties;
-  int pos;
-  int k, r;
-  int moves_played;
-  int potential_sekis[BOARDMAX];
+  int lib1;
+  int lib2;
 
-  /* Find isolated two-stone strings which might be involved in the
-   * kind of seki described in the comments.
-   */
-  memset(potential_sekis, 0, sizeof(potential_sekis));
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    int isolated = 1;
-    int stones[2];
-    int pos2;
-    
-    if (board[pos] != color
-	|| find_origin(pos) != pos
-	|| countstones(pos) != 2)
-      continue;
-    
-    findstones(pos, 2, stones);
-    for (k = 0; k < 2 && isolated; k++) {
-      for (r = 0; r < 8 && isolated; r++) {
-	pos2 = stones[k] + delta[r];
-	if (!ON_BOARD(pos2)
-	    || (board[pos2] == color
-		&& !same_string(pos, pos2)))
-	  isolated = 0;
-      }
-    }
+  /* Try to play the move. */
+  if (!trymove(move, color, "send_two_return_one-A", NO_MOVE))
+    return 0;
 
-    if (isolated) {
-      potential_sekis[stones[0]] = 1;
-      potential_sekis[stones[1]] = 1;
-    }
-  }
-  
-  moves_played = capture_non_invincible_strings(color, potential_sekis);
-  /* The strings still remaining except those marked in
-   * potential_sekis[] are uncapturable. Now see which opponent
-   * strings can survive.
-   *
-   * 1. Play opponent stones on all liberties of the unconditionally
-   *    alive strings except where illegal.
-   */
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (board[pos] != color || potential_sekis[pos] || find_origin(pos) != pos)
-      continue;
-      
-    /* Play as many liberties as we can. */
-    liberties = findlib(pos, MAXLIBS, libs);
-    for (k = 0; k < liberties; k++) {
-      if (trymove(libs[k], other, "unconditional_life", pos))
-	moves_played++;
-    }
-  }
-
-  /* 2. Recursively extend opponent strings in atari, except where this
-   *    would be suicide.
-   */
-  found_one = 1;
-  while (found_one) {
-    /* Nothing found so far in this turn of the loop. */
-    found_one = 0;
-
-    for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-      if (board[pos] != other || countlib(pos) > 1)
-	continue;
-	
-      /* Try to extend the string at (m, n). */
-      findlib(pos, 1, libs);
-      if (trymove(libs[0], other, "unconditional_life", pos)) {
-	moves_played++;
-	found_one = 1;
-      }
-    }
-  }
-
-  /* Now see whether there are any significant sekis on the board. */
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (!potential_sekis[pos]
-	|| board[pos] == EMPTY
-	|| find_origin(pos) != pos)
-      continue;
-    for (r = 0; r < 4; r++) {
-      int up = delta[r];
-      int right = delta[(r + 1) % 4];
-      int locally_played_moves = 0;
-      if (board[pos + up] != color
-	  || board[pos + up + up] != EMPTY
-	  || board[pos - up] != EMPTY)
-	continue;
-      for (k = 0; k < 2; k++) {
-	if (k == 1)
-	  right = -right;
-	if (board[pos + right] != EMPTY || board[pos + up - right] != EMPTY)
-	  continue;
-	if (board[pos - right] == EMPTY
-	    && trymove(pos - right, other, "unconditional_life", pos))
-	  locally_played_moves++;
-	if (board[pos + up + right] == EMPTY
-	    && trymove(pos + up + right, other, "unconditional_life", pos))
-	  locally_played_moves++;
-	if (board[pos - right] == other && board[pos + up + right] == other
-	    && same_string(pos - right, pos + up + right)) {
-	  /* This is a critical seki. Extend the string with one stone
-           * in an arbitrary direction to break the seki.
-	   */
-	  while (locally_played_moves > 0) {
-	    popgo();
-	    locally_played_moves--;
-	  }
-	  trymove(pos - up, color, "unconditional_life", pos);
-	  moves_played++;
-	  break;
-	}
-	else {
-	  while (locally_played_moves > 0) {
-	    popgo();
-	    locally_played_moves--;
-	  }
-	}
-      }
-      if (countstones(pos) > 2)
-	break;
-    }
-  }
-
-  /* Capture the strings involved in potential sekis. */
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (!potential_sekis[pos] || board[pos] == EMPTY)
-      continue;
-    /* Play as many liberties as we can. */
-    liberties = findlib(pos, MAXLIBS, libs);
-    for (k = 0; k < liberties; k++) {
-      if (trymove(libs[k], other, "unconditional_life", pos))
-	moves_played++;
-    }
-  }
-    
-
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    int apos;
-    int bpos;
-    int aopen, bopen;
-    int alib, blib;
-    if (board[pos] != other || countlib(pos) != 2)
-      continue;
-    findlib(pos, 2, libs);
-    apos = libs[0];
-    bpos = libs[1];
-    if (abs(I(apos) - I(bpos)) + abs(J(apos) - J(bpos)) != 1)
-      continue;
-    
-    /* Only two liberties and these are adjacent. Play one. We want
-     * to maximize the number of open liberties. In this particular
-     * situation we can count this with approxlib for the opposite
-     * color. If the number of open liberties is the same, we
-     * maximize the total number of obtained liberties.
-     * Two relevant positions:
-     *
-     * |XXX. 
-     * |OOXX    |XXXXXXX
-     * |O.OX    |OOXOOOX
-     * |..OX    |..OO.OX
-     * +----    +-------
-     */
-    aopen = approxlib(apos, color, 4, NULL);
-    bopen = approxlib(bpos, color, 4, NULL);
-    alib  = approxlib(apos, other, 4, NULL);
-    blib  = approxlib(bpos, other, 4, NULL);
-    
-    if (aopen > bopen || (aopen == bopen && alib >= blib)) {
-      trymove(apos, other, "unconditional_life", pos);
-      moves_played++;
-    }
-    else {
-      trymove(bpos, other, "unconditional_life", pos);
-      moves_played++;
-    }
-  }
-  
-  /* Identify unconditionally alive stones and unconditional territory. */
-  memset(unconditional_territory, 0, sizeof(int) * BOARDMAX);
-  for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
-    if (board[pos] == color && !potential_sekis[pos]) {
-      unconditional_territory[pos] = 1;
-      if (find_origin(pos) == pos) {
-	liberties = findlib(pos, MAXLIBS, libs);
-	for (k = 0; k < liberties; k++)
-	  unconditional_territory[libs[k]] = 2;
-      }
-    }
-    else if (board[pos] == other && countlib(pos) == 1) {
-      unconditional_territory[pos] = 2;
-      findlib(pos, 1, libs);
-      unconditional_territory[libs[0]] = 2;
-    }
-  }
-  
-  /* Take back all moves. */
-  while (moves_played > 0) {
+  /* Does the move leave exactly two stones in atari? */
+  if (findlib(move, 1, &lib1) > 1 || countstones(move) != 2) {
     popgo();
-    moves_played--;
+    return 0;
   }
+
+  /* Let the opponent capture. This move is guaranteed to be legal. */
+  trymove(lib1, other, "send_two_return_one-A", NO_MOVE);
+
+  /* Is the capturing string single and in atari? */
+  if (findlib(lib1, 1, &lib2) > 1 || countstones(lib1) > 1) {
+    popgo();
+    popgo();
+    return 0;
+  }
+
+  popgo();
+  popgo();
+
+  /* The move to capture back is in lib2. This must be one of the two
+   * captured stones, either the move or the adjacent stone. If it is
+   * the same as the first move, the previous position is not repeated
+   * by capturing back, so we return false.
+   */
+  if (lib2 == move)
+    return 0;
+
+  /* The move matches the shape. */
+  return 1;
 }
 
 
@@ -1889,7 +1467,7 @@ who_wins(int color, FILE *outfile)
   }
 #endif
 
-  result = estimate_score(NULL, NULL);
+  result = (white_score + black_score)/2.0;
   if (result == 0.0)
     fprintf(outfile, "Result: jigo   ");
   else
