@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -697,10 +697,8 @@ compute_unconditional_status()
   int unconditional_territory[BOARDMAX];
   int pos;
   int color;
-  int other;
   
   for (color = WHITE; color <= BLACK; color++) {
-    other = OTHER_COLOR(color);
     unconditional_life(unconditional_territory, color);
 
     for (pos = BOARDMIN; pos < BOARDMAX; pos++) {
@@ -877,15 +875,12 @@ find_worm_threats()
   int k;
   int l;
   int color;
-  int other;
   
   for (str = BOARDMIN; str < BOARDMAX; str++) {
     color = board[str];
     if (!IS_STONE(color) || !is_worm_origin(str, str))
       continue;
 
-    other = OTHER_COLOR(color);
-
     /* 1. Start with finding attack threats. */
     /* Only try those worms that have no attack. */
     if (worm[str].attack_codes[0] == 0) {
