@@ -6,7 +6,7 @@
  * files gtp.c and gtp.h are licensed under less restrictive     *
  * terms than the rest of GNU Go.                                *
  *                                                               *
- * Copyright 2001, 2002, 2003 and 2004                           *
+ * Copyright 2001, 2002, 2003, 2004 and 2005                     *
  * by the Free Software Foundation.                              *
  *                                                               *
  * Permission is hereby granted, free of charge, to any person   *
@@ -38,6 +38,9 @@
  * without prior written authorization of the copyright holder.  *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
+/* NOTE: GNU Go specific, workarounds dumb Windows sockets. */
+#include "winsocket.h"
+
 #include <stdarg.h>
 #include <stdio.h>
 
@@ -65,8 +68,8 @@ struct gtp_command {
   gtp_fn_ptr function;
 };
 
-void gtp_main_loop(struct gtp_command commands[], FILE *gtp_input,
-		   FILE *gtp_dump_commands);
+void gtp_main_loop(struct gtp_command commands[],
+		   FILE *gtp_input, FILE *gtp_output, FILE *gtp_dump_commands);
 void gtp_internal_set_boardsize(int size);
 void gtp_set_vertex_transform_hooks(gtp_transform_ptr in,
 				    gtp_transform_ptr out);
@@ -83,6 +86,8 @@ int gtp_decode_move(char *s, int *color, int *i, int *j);
 void gtp_print_vertices(int n, int movei[], int movej[]);
 void gtp_print_vertex(int i, int j);
 
+extern FILE *gtp_output_file;
+
 /*
  * Local Variables:
  * tab-width: 8
