@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -49,6 +49,7 @@
 
 #include "gg-getopt.h"
 #include "gg_utils.h"
+#include "winsocket.h"
 
 #include "interface.h"
 #include "sgftree.h"
@@ -60,6 +61,14 @@ static void show_help(void);
 static void show_debug_help(void);
 static void show_debug_flags(void);
 
+static void socket_connect_to(const char *host_name, unsigned int port,
+			      FILE **input_file, FILE **output_file);
+static void socket_listen_at(const char *host_name, unsigned int port,
+			     FILE **input_file, FILE **output_file);
+static void socket_close_connection(FILE *input_file, FILE *output_file);
+static void socket_stop_listening(FILE *input_file, FILE *output_file);
+
+
 /* long options which have no short form */
 enum {OPT_BOARDSIZE = 127,
       OPT_HANDICAPSTONES,
@@ -74,6 +83,8 @@ enum {OPT_BOARDSIZE = 127,
       OPT_OUTFILE, 
       OPT_QUIET,
       OPT_GTP_INPUT,
+      OPT_GTP_CONNECT,
+      OPT_GTP_LISTEN,
       OPT_GTP_DUMP_COMMANDS,
       OPT_GTP_INITIAL_ORIENTATION,
       OPT_GTP_VERSION,
@@ -86,7 +97,9 @@ enum {OPT_BOARDSIZE = 127,
       OPT_DECIDE_SEMEAI,
       OPT_DECIDE_SURROUNDED,
       OPT_DECIDE_TACTICAL_SEMEAI,
+      OPT_DECIDE_ORACLE,
       OPT_EXPERIMENTAL_SEMEAI,
+      OPT_EXPERIMENTAL_OWL_EXT,
       OPT_SEMEAI_NODE_LIMIT,
       OPT_EXPERIMENTAL_CONNECTIONS,
       OPT_EXPERIMENTAL_INFLUENCE,
@@ -149,7 +162,6 @@ enum {OPT_BOARDSIZE = 127,
 enum mode {
   MODE_UNKNOWN = 0,
   MODE_ASCII,
-  MODE_ASCII_EMACS,
   MODE_GTP,
   MODE_GMP,
   MODE_SGMP,
@@ -168,7 +180,8 @@ enum mode {
   MODE_DECIDE_POSITION,
   MODE_DECIDE_EYE,
   MODE_DECIDE_COMBINATION,
-  MODE_DECIDE_SURROUNDED
+  MODE_DECIDE_SURROUNDED,
+  MODE_DECIDE_ORACLE
 };
 
 
@@ -186,6 +199,8 @@ static struct gg_option const long_options[] =
   {"quiet",          no_argument,       0, OPT_QUIET},
   {"silent",         no_argument,       0, OPT_QUIET},
   {"gtp-input",      required_argument, 0, OPT_GTP_INPUT},
+  {"gtp-connect",    required_argument, 0, OPT_GTP_CONNECT},
+  {"gtp-listen",     required_argument, 0, OPT_GTP_LISTEN},
   {"gtp-dump-commands", required_argument, 0, OPT_GTP_DUMP_COMMANDS},
   {"orientation",    required_argument, 0, OPT_GTP_INITIAL_ORIENTATION},
   {"gtp-initial-orientation",
@@ -232,6 +247,7 @@ static struct gg_option const long_options[] =
   {"chinese-rules",  no_argument,       0, OPT_CHINESE_RULES},
   {"japanese-rules", no_argument,       0, OPT_JAPANESE_RULES},
   {"experimental-semeai",  no_argument, 0, OPT_EXPERIMENTAL_SEMEAI},
+  {"experimental-owl-ext",  no_argument, 0, OPT_EXPERIMENTAL_OWL_EXT},
   {"semeai-node-limit",   required_argument, 0, OPT_SEMEAI_NODE_LIMIT},
   {"experimental-connections",  no_argument, 0, OPT_EXPERIMENTAL_CONNECTIONS},
   {"standard-connections",  no_argument, 0, OPT_STANDARD_CONNECTIONS},
@@ -266,6 +282,7 @@ static struct gg_option const long_options[] =
   {"decide-surrounded",  required_argument, 0, OPT_DECIDE_SURROUNDED},
   {"decide-eye",     required_argument, 0, OPT_DECIDE_EYE},
   {"decide-combination", no_argument,   0, OPT_DECIDE_COMBINATION},
+  {"decide-oracle",  no_argument,       0, OPT_DECIDE_ORACLE},
   {"nofusekidb",     no_argument,       0, OPT_NOFUSEKIDB},
   {"nofuseki",       no_argument,       0, OPT_NOFUSEKI},
   {"nojosekidb",     no_argument,       0, OPT_NOJOSEKIDB},
@@ -304,6 +321,8 @@ main(int argc, char *argv[])
   char *outflags = NULL;
   char *gtpfile = NULL;
   char *gtp_dump_commands_file = NULL;
+  int gtp_tcp_ip_mode = 0;
+  char *gtp_tcp_ip_address = NULL;
   
   char *printsgffile = NULL;
   
@@ -312,8 +331,6 @@ main(int argc, char *argv[])
   char debuginfluence_move[4] = "\0";
   
   int benchmark = 0;  /* benchmarking mode (-b) */
-  FILE *gtp_input_FILE;
-  FILE *gtp_dump_commands_FILE = NULL;
   FILE *output_check;
   int orientation = 0;
 
@@ -358,6 +375,7 @@ main(int argc, char *argv[])
     owl_threats = 1;
   else
     owl_threats = 0;
+  experimental_owl_ext = EXPERIMENTAL_OWL_EXT;
   experimental_connections = EXPERIMENTAL_CONNECTIONS;
   cosmic_gnugo = COSMIC_GNUGO;
   large_scale = LARGE_SCALE;
@@ -435,11 +453,25 @@ main(int argc, char *argv[])
       case OPT_QUIET:
 	quiet = 1;
 	break;
-	
+
       case OPT_GTP_INPUT:
-	gtpfile = gg_optarg;
+      case OPT_GTP_CONNECT:
+      case OPT_GTP_LISTEN:
+	if (gtp_tcp_ip_mode != 0 || gtpfile != NULL) {
+	  fprintf(stderr, ("Options `--gtp-input', `--gtp-connect' and `--gtp-listen' "
+			   "are mutually-exclusive\n"));
+	  exit(EXIT_FAILURE);
+	}
+
+	if (i == OPT_GTP_INPUT)
+	  gtpfile = gg_optarg;
+	else {
+	  gtp_tcp_ip_mode = i;
+	  gtp_tcp_ip_address = gg_optarg;
+	}
+
 	break;
-	
+
       case OPT_GTP_DUMP_COMMANDS:
 	gtp_dump_commands_file = gg_optarg;
 	break;
@@ -447,7 +479,7 @@ main(int argc, char *argv[])
       case OPT_GTP_INITIAL_ORIENTATION:
 	orientation = atoi(gg_optarg);
 	if (orientation < 0 || orientation > 7) {
-	  fprintf(stderr, "Illegal orientation: %d.\n", orientation);
+	  fprintf(stderr, "Invalid orientation: %d.\n", orientation);
 	  fprintf(stderr, "Try `gnugo --help' for more information.\n");
 	  exit(EXIT_FAILURE);
 	}
@@ -484,10 +516,10 @@ main(int argc, char *argv[])
 		  "configure option enabled: owl threats\n");
 	if (RESIGNATION_ALLOWED)
 	  fprintf(stdout,
-		  "configure option enabled: resignation allowed\n");
+		  "configure option enabled: resination allowed\n");
 	if (ORACLE)
 	  fprintf(stdout,
-		  "configure option enabled: metamachine\n");
+		  "configure option enabled: oracle\n");
 	fprintf(stdout,
 		"Owl node limit: %d\n", OWL_NODE_LIMIT);
 	fprintf(stdout,
@@ -511,7 +543,7 @@ main(int argc, char *argv[])
 	  int handicap = atoi(gg_optarg);
 	  
 	  if (handicap < 0 || handicap > MAX_HANDICAP) {
-	    fprintf(stderr, "Illegal handicap: %d.\n", handicap);
+	    fprintf(stderr, "Unsupported handicap: %d.\n", handicap);
 	    fprintf(stderr, "Try `gnugo --help' for more information.\n");
 	    exit(EXIT_FAILURE);
 	  }
@@ -524,7 +556,7 @@ main(int argc, char *argv[])
 	  int boardsize = atoi(gg_optarg);
 	  
 	  if (boardsize < MIN_BOARD || boardsize > MAX_BOARD) {
-	    fprintf(stderr, "Illegal board size: %d.\n", boardsize);
+	    fprintf(stderr, "Unsupported board size: %d.\n", boardsize);
 	    fprintf(stderr, "Try `gnugo --help' for more information.\n");
 	    exit(EXIT_FAILURE);
 	  }
@@ -560,6 +592,10 @@ main(int argc, char *argv[])
 	chinese_rules = 0;
 	break;
 
+      case OPT_EXPERIMENTAL_OWL_EXT:
+	experimental_owl_ext = 1;
+	break;
+
       case OPT_SEMEAI_NODE_LIMIT:
 	semeai_node_limit = atoi(gg_optarg);
 	break;
@@ -623,8 +659,6 @@ main(int argc, char *argv[])
       case OPT_MODE: 
 	if (strcmp(gg_optarg, "ascii") == 0)
 	  playmode = MODE_ASCII;
-	else if (strcmp(gg_optarg, "emacs") == 0)
-	  playmode = MODE_ASCII_EMACS;
 	else if (strcmp(gg_optarg, "gtp") == 0)
 	  playmode = MODE_GTP;
 	else if (strcmp(gg_optarg, "gmp") == 0)
@@ -745,6 +779,10 @@ main(int argc, char *argv[])
 	playmode = MODE_DECIDE_SURROUNDED;
 	break;
 	
+      case OPT_DECIDE_ORACLE:
+	playmode = MODE_DECIDE_ORACLE;
+	break;
+	
       case OPT_BRANCH_DEPTH:
 	mandated_branch_depth = atoi(gg_optarg);
 	break;
@@ -985,8 +1023,7 @@ main(int argc, char *argv[])
       exit(EXIT_FAILURE);
     }
     
-    gameinfo_play_sgftree_rot(&gameinfo, &sgftree, untilstring,
-                             orientation);
+    gameinfo_play_sgftree_rot(&gameinfo, &sgftree, untilstring, orientation);
   }
   else
   /* Initialize and empty sgf tree if there was no infile. */
@@ -1248,14 +1285,10 @@ main(int argc, char *argv[])
 
   case MODE_DECIDE_POSITION:
     {
-      int color;
       if (!infilename) {
 	fprintf(stderr, "gnugo: --decide-position must be used with -l\n");
 	return EXIT_FAILURE;
       }
-      color = gameinfo.to_move;
-      if (mandated_color != EMPTY)
-	color = mandated_color;
       decide_position();
     }
     break;
@@ -1308,36 +1341,85 @@ main(int argc, char *argv[])
       break;
     }
 
-  case MODE_GTP:  
-    if (gtpfile != NULL) {
-      gtp_input_FILE = fopen(gtpfile, "r");
-      if (gtp_input_FILE == NULL) {
-	fprintf(stderr, "gnugo: Cannot open file %s\n", gtpfile);
-	return EXIT_FAILURE;
-      } 
-    }
-    else
-      gtp_input_FILE = stdin;
+#if ORACLE
+  case MODE_DECIDE_ORACLE:
+    {
+      if (mandated_color != EMPTY)
+	gameinfo.to_move = mandated_color;
+      
+      if (!infilename) {
+	fprintf(stderr, "You must use -l infile with load and analyze mode.\n");
+	exit(EXIT_FAILURE);
+      }
 
-    if (gtp_dump_commands_file != NULL) {
-      gtp_dump_commands_FILE = fopen(gtp_dump_commands_file, "w");
-      if (gtp_dump_commands_FILE == NULL) {
-	fprintf(stderr, "gnugo: Cannot open file %s\n",
-		gtp_dump_commands_file);
-	return EXIT_FAILURE;
-      } 
+      decide_oracle(&gameinfo, infilename, untilstring);
+      break;
+    }
+#endif
+
+  case MODE_GTP:
+    {
+      FILE *gtp_input_FILE = stdin;
+      FILE *gtp_output_FILE = stdout;
+      FILE *gtp_dump_commands_FILE = NULL;
+
+      if (gtpfile != NULL) {
+	gtp_input_FILE = fopen(gtpfile, "r");
+	if (gtp_input_FILE == NULL) {
+	  fprintf(stderr, "gnugo: Cannot open file %s\n", gtpfile);
+	  return EXIT_FAILURE;
+	}
+      }
+      else if (gtp_tcp_ip_mode != 0) {
+	unsigned int port = 65536;
+	char *port_string = strchr(gtp_tcp_ip_address, ':');
+	const char *host_name = NULL;
+
+	if (port_string) {
+	  host_name = gtp_tcp_ip_address;
+
+	  *port_string++ = 0;
+	  sscanf(port_string, "%u", &port);
+	}
+	else
+	  sscanf(gtp_tcp_ip_address, "%u", &port);
+
+	if (port > 65535) {
+	  fprintf(stderr, "A valid TCP/IP port number expected\n");
+	  exit(EXIT_FAILURE);
+	}
+
+	if (gtp_tcp_ip_mode == OPT_GTP_CONNECT) {
+	  socket_connect_to(host_name, port,
+			    &gtp_input_FILE, &gtp_output_FILE);
+	}
+	else {
+	  socket_listen_at(host_name, port,
+			   &gtp_input_FILE, &gtp_output_FILE);
+	}
+      }
+
+      if (gtp_dump_commands_file != NULL) {
+	gtp_dump_commands_FILE = fopen(gtp_dump_commands_file, "w");
+	if (gtp_dump_commands_FILE == NULL) {
+	  fprintf(stderr, "gnugo: Cannot open file %s\n",
+		  gtp_dump_commands_file);
+	  return EXIT_FAILURE;
+	}
+      }
+
+      play_gtp(gtp_input_FILE, gtp_output_FILE, gtp_dump_commands_FILE,
+	       orientation);
+
+      if (gtp_dump_commands_FILE)
+	fclose(gtp_dump_commands_FILE);
+
+      if (gtp_tcp_ip_mode == OPT_GTP_CONNECT)
+	socket_close_connection(gtp_input_FILE, gtp_output_FILE);
+      else if (gtp_tcp_ip_mode == OPT_GTP_LISTEN)
+	socket_stop_listening(gtp_input_FILE, gtp_output_FILE);
     }
 
-    play_gtp(gtp_input_FILE, gtp_dump_commands_FILE, orientation);
-    if (gtp_dump_commands_FILE)
-      fclose(gtp_dump_commands_FILE);
-    break;
-
-  case MODE_ASCII_EMACS:  
-    if (mandated_color != EMPTY)
-      gameinfo.computer_player = OTHER_COLOR(mandated_color);
-
-    play_ascii_emacs(&sgftree, &gameinfo, infilename, untilstring);
     break;
 
   case MODE_ASCII:  
@@ -1351,32 +1433,16 @@ main(int argc, char *argv[])
       show_copyright();
     }
   
-#if ORACLE
-    if (metamachine) {
-      summon_oracle();
-      oracle_loadsgf(infilename, untilstring);
-    }
-#endif
-
     play_ascii(&sgftree, &gameinfo, infilename, untilstring);
-
-#if ORACLE
-    dismiss_oracle();
-#endif
-
     break;
   }
   
-#if ORACLE
-  dismiss_oracle();
-#endif
-
   if (profile_patterns)
     report_pattern_profiling();
 
   clock_report_autolevel(NULL, gameinfo.computer_player);
-  sgfFreeNode(gameinfo.game_record.root);
- 
+  sgfFreeNode(sgftree.root); 
+
   return 0;
 }  /* end main */
 
@@ -1420,8 +1486,14 @@ Main Options:\n\
        --quiet           Don't print copyright and informational messages\n\
        --level <amount>  strength (default %d)\n\
        --never-resign    Forbid GNU Go to resign\n\
-       --resign-allowed	 Allow resignation (default)\n\
+       --resign-allowed  Allow resignation (default)\n\
        --gtp-input <file>Read gtp commands from file instead of stdin\n\
+       --gtp-connect [HOST:]PORT\n\
+                         Connect to given host (127.0.0.1 if omitted) and port\n\
+                         and receive GTP commands on the established connection\n\
+       --gtp-listen [HOST:]PORT\n\
+                         Wait for the first TCP/IP connection on the given port\n\
+                         (if HOST is specified, only to that host)\n\
    -l, --infile <file>   Load name sgf file\n\
    -L, --until <move>    Stop loading just before move is played. <move>\n\
                          can be the move number or location (eg L10).\n\
@@ -1478,7 +1550,7 @@ Informative Output:\n\
 
 
 #define COPYRIGHT \
-"Copyright 1999, 2000, 2001, 2002, 2003 and 2004 by the Free Software\n\
+"Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005 by the Free Software\n\
 Foundation, Inc. See http://www.gnu.org/software/gnugo/ or contact\n\
 gnugo@gnu.org for information about GNU Go. GNU Go comes with NO WARRANTY to\n\
 the extent permitted by law. This program is free software; you can\n\
@@ -1613,6 +1685,258 @@ show_copyright(void)
 }
 
 
+#if ENABLE_SOCKET_SUPPORT
+
+
+#if !defined(_WIN32) && !defined(_WIN32_WCE)
+
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <netdb.h>
+
+#define closesocket	close
+#define init_sockets()
+
+#else	/* on Windows */
+
+
+#include <winsocket.h>
+
+
+static void
+init_sockets(void)
+{
+  WSADATA data;
+  WORD version = MAKEWORD(1, 1);
+
+  if (WSAStartup(version, &data) != NO_ERROR) {
+    fprintf(stderr, "WSAStartup() failed with error %d\n", WSAGetLastError());
+    exit(EXIT_FAILURE);
+  }
+}
+
+
+#endif	/* on Windows */
+
+
+static void
+socket_connect_to(const char *host_name, unsigned int port,
+		  FILE **input_file, FILE **output_file)
+{
+  struct sockaddr_in address;
+  int connection_socket;
+  struct hostent *host_data;
+  char **address_pointer;
+
+  init_sockets();
+
+  if (!host_name)
+    host_name = "127.0.0.1";
+
+  host_data = gethostbyname(host_name);
+  if (!host_data
+      || host_data->h_addrtype != AF_INET
+      || host_data->h_length != sizeof address.sin_addr) {
+    fprintf(stderr, "Failed to resolve host name `%s'\n", host_name);
+    exit(EXIT_FAILURE);
+  }
+
+  connection_socket = socket(PF_INET, SOCK_STREAM, 0);
+  if (connection_socket == -1) {
+    fprintf(stderr, "Unexpected error: failed to create a socket\n");
+    exit(EXIT_FAILURE);
+  }
+
+  address.sin_family = AF_INET;
+  address.sin_port = htons(port);
+
+  for (address_pointer = host_data->h_addr_list; *address_pointer;
+       address_pointer++) {
+    memcpy(&address.sin_addr, *address_pointer, sizeof address.sin_addr);
+    if (connect(connection_socket, (struct sockaddr *) &address,
+		sizeof address) != -1)
+      break;
+  }
+
+  if (! *address_pointer) {
+    fprintf(stderr, "Failed to connect to %s:%d\n", host_data->h_name, port);
+    closesocket(connection_socket);
+    exit(EXIT_FAILURE);
+  }
+
+#if !USE_WINDOWS_SOCKET_CLUDGE
+
+  *input_file  = fdopen(connection_socket, "r");
+  *output_file = fdopen(dup(connection_socket), "w");
+
+#else	/* USE_WINDOWS_SOCKET_CLUDGE */
+
+  winsocket_activate(connection_socket);
+
+  *input_file  = NULL;
+  *output_file = NULL;
+
+#endif	/* USE_WINDOWS_SOCKET_CLUDGE */
+}
+
+
+static void
+socket_listen_at(const char *host_name, unsigned int port,
+		 FILE **input_file, FILE **output_file)
+{
+  struct sockaddr_in address;
+  int listening_socket;
+  int connection_socket;
+
+  init_sockets();
+
+  if (host_name) {
+    struct hostent *host_data;
+
+    host_data = gethostbyname(host_name);
+    if (!host_data
+	|| host_data->h_addrtype != AF_INET
+	|| host_data->h_length != sizeof address.sin_addr) {
+      fprintf(stderr, "Failed to resolve host name `%s'\n", host_name);
+      exit(EXIT_FAILURE);
+    }
+
+    host_name = host_data->h_name;
+    memcpy(&address.sin_addr, host_data->h_addr_list[0],
+	   sizeof address.sin_addr);
+  }
+  else
+    address.sin_addr.s_addr = htonl(INADDR_ANY);
+
+  listening_socket = socket(PF_INET, SOCK_STREAM, 0);
+  if (listening_socket == -1) {
+    fprintf(stderr, "Unexpected error: failed to create a socket\n");
+    exit(EXIT_FAILURE);
+  }
+
+  address.sin_family = AF_INET;
+  address.sin_port = htons(port);
+
+  if (verbose) {
+    if (host_name) {
+      fprintf(stderr, "Waiting for a connection on %s:%d...\n",
+	      host_name, port);
+    }
+    else
+      fprintf(stderr, "Waiting for a connection on port %d...\n", port);
+  }
+
+  if (bind(listening_socket,
+	   (struct sockaddr *) &address, sizeof address) == -1
+      || listen(listening_socket, 0) == -1
+      || (connection_socket = accept(listening_socket, NULL, NULL)) == -1) {
+    if (host_name)
+      fprintf(stderr, "Failed to listen on %s:%d\n", host_name, port);
+    else
+      fprintf(stderr, "Failed to listen on port %d\n", port);
+
+    closesocket(listening_socket);
+    exit(EXIT_FAILURE);
+  }
+
+  closesocket(listening_socket);
+
+#if !USE_WINDOWS_SOCKET_CLUDGE
+
+  *input_file  = fdopen(connection_socket, "r");
+  *output_file = fdopen(dup(connection_socket), "w");
+
+#else	/* USE_WINDOWS_SOCKET_CLUDGE */
+
+  winsocket_activate(connection_socket);
+
+  *input_file  = NULL;
+  *output_file = NULL;
+
+#endif	/* USE_WINDOWS_SOCKET_CLUDGE */
+}
+
+
+static void
+socket_close_connection(FILE *input_file, FILE *output_file)
+{
+  /* When connecting, we close the socket first. */
+  fclose(input_file);
+  fclose(output_file);
+}
+
+
+static void
+socket_stop_listening(FILE *input_file, FILE *output_file)
+{
+  int buffer[0x1000];
+
+  if (verbose)
+    fprintf(stderr, "Waiting for the client to disconnect...\n");
+
+  /* When listening, we wait for the client to disconnect first.
+   * Otherwise, socket doesn't get released properly.
+   */
+  do
+    fread(buffer, sizeof buffer, 1, input_file);
+  while (!feof(input_file));
+
+  fclose(input_file);
+  fclose(output_file);
+}
+
+
+#else  /* not ENABLE_SOCKET_SUPPORT */
+
+
+static void
+socket_connect_to(const char *host_name, unsigned int port,
+		  FILE **input_file, FILE **output_file)
+{
+  UNUSED(host_name);
+  UNUSED(port);
+  UNUSED(input_file);
+  UNUSED(output_file);
+
+  fprintf(stderr, "GNU Go was compiled without socket support, unable to connect\n");
+  exit(EXIT_FAILURE);
+}
+
+
+static void
+socket_listen_at(const char *host_name, unsigned int port,
+		 FILE **input_file, FILE **output_file)
+{
+  UNUSED(host_name);
+  UNUSED(port);
+  UNUSED(input_file);
+  UNUSED(output_file);
+
+  fprintf(stderr, "GNU Go was compiled without socket support, unable to listen\n");
+  exit(EXIT_FAILURE);
+}
+
+
+static void
+socket_close_connection(FILE *input_file, FILE *output_file)
+{
+  UNUSED(input_file);
+  UNUSED(output_file);
+}
+
+
+static void
+socket_stop_listening(FILE *input_file, FILE *output_file)
+{
+  UNUSED(input_file);
+  UNUSED(output_file);
+}
+
+
+#endif	/* not ENABLE_SOCKET_SUPPORT */
+
+
 /*
  * Local Variables:
  * tab-width: 8
