@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -27,6 +27,11 @@
 #include <string.h>
 #include <ctype.h>
 
+#ifdef READLINE
+#include <readline/readline.h>
+#include <readline/history.h>
+#endif
+
 #include "liberty.h"
 #include "interface.h"
 #include "sgftree.h"
@@ -46,10 +51,7 @@ showterri        display territory\n\
 /* some options for the ascii interface */
 static int opt_showboard = 1;
 static int showdead = 0;
-static int emacs = 0;
 static SGFTree sgftree;
-static int last_move_i;      /* The position of the last move */
-static int last_move_j;      /* -""-                          */
 static int resignation_allowed;
 
 /* Unreasonable score used to detect missing information. */
@@ -57,6 +59,7 @@ static int resignation_allowed;
 /* Keep track of the score estimated before the last computer move. */
 static int current_score_estimate = NO_SCORE;
 
+static void do_play_ascii(Gameinfo *gameinfo);
 static int ascii_endgame(Gameinfo *gameinfo, int reason);
 static void ascii_count(Gameinfo *gameinfo);
 static void showcapture(char *line);
@@ -190,6 +193,7 @@ ascii_showboard(void)
   int last_pos_was_move;
   int pos_is_move;
   int dead;
+  int last_move = get_last_move();
   
   make_letterbar(board_size, letterbar);
   set_handicap_spots(board_size);
@@ -213,14 +217,22 @@ ascii_showboard(void)
   printf("\n");
 
   fflush(stdout);
-  printf("%s%s\n", (emacs ? "EMACS1\n" : ""), letterbar);
+  printf("%s", letterbar);
+
+  if (get_last_player() != EMPTY) {
+    gfprintf(stdout, "        Last move: %s %1m",
+	     get_last_player() == WHITE ? "White" : "Black",
+	     last_move);
+  }
+
+  printf("\n");
   fflush(stdout);
   
   for (i = 0; i < board_size; i++) {
     printf(" %2d", board_size - i);
     last_pos_was_move = 0;
     for (j = 0; j < board_size; j++) {
-      if (last_move_i == i && last_move_j == j)
+      if (POS(i, j) == last_move)
 	pos_is_move = 128;
       else
 	pos_is_move = 0;
@@ -444,15 +456,16 @@ static int
 computer_move(Gameinfo *gameinfo, int *passes)
 {
   int i, j;
-  int move_val;
+  float move_value;
+  int resign;
   int resignation_declined = 0;
   float upper_bound, lower_bound;
 
   init_sgf(gameinfo);
 
   /* Generate computer move. */
-  move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
-  if (resignation_allowed && move_val < 0.0 && ON_BOARD2(i, j)) {
+  move_value = gnugo_genmove(&i, &j, gameinfo->to_move, &resign);
+  if (resignation_allowed && resign) {
     int state = ascii_endgame(gameinfo, 2);
     if (state != -1)
       return state;
@@ -460,16 +473,12 @@ computer_move(Gameinfo *gameinfo, int *passes)
     /* The opponent declined resignation. Remember not to resign again. */
     resignation_allowed = 0;
     resignation_declined = 1;
-    move_val = -move_val;
   }
 
   if (showscore) {
     gnugo_estimate_score(&upper_bound, &lower_bound);
     current_score_estimate = (int) ((lower_bound + upper_bound) / 2.0);
   }
-    
-  last_move_i = i;
-  last_move_j = j;
   
   mprintf("%s(%d): %m\n", color_to_string(gameinfo->to_move),
 	  movenum+1, i, j);
@@ -479,7 +488,7 @@ computer_move(Gameinfo *gameinfo, int *passes)
     *passes = 0;
 
   gnugo_play_move(i, j, gameinfo->to_move);
-  sgffile_add_debuginfo(sgftree.lastnode, move_val);
+  sgffile_add_debuginfo(sgftree.lastnode, move_value);
   sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
   if (resignation_declined)
     sgftreeAddComment(&sgftree, "GNU Go resignation was declined");
@@ -513,14 +522,11 @@ do_move(Gameinfo *gameinfo, char *command, int *passes, int force)
   TRACE("\nyour move: %m\n\n", i, j);
   init_sgf(gameinfo);
   gnugo_play_move(i, j, gameinfo->to_move);
-  sgffile_add_debuginfo(sgftree.lastnode, 0);
+  sgffile_add_debuginfo(sgftree.lastnode, 0.0);
   sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
   sgffile_output(&sgftree);
-
-  last_move_i = i;
-  last_move_j = j;
   
-  if (opt_showboard && !emacs) {
+  if (opt_showboard) {
     ascii_showboard();
     printf("GNU Go is thinking...\n");
   }
@@ -547,7 +553,7 @@ do_pass(Gameinfo *gameinfo, int *passes, int force)
   (*passes)++;
   init_sgf(gameinfo);
   gnugo_play_move(-1, -1, gameinfo->to_move);
-  sgffile_add_debuginfo(sgftree.lastnode, 0);
+  sgffile_add_debuginfo(sgftree.lastnode, 0.0);
   sgftreeAddPlay(&sgftree, gameinfo->to_move, -1, -1);
   sgffile_output(&sgftree);
 
@@ -569,16 +575,7 @@ do_pass(Gameinfo *gameinfo, int *passes, int force)
 void
 play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 {
-  int m, num;
   int sz;
-  float fnum;
-  int passes = 0;  /* two passes and its over */
-  int tmp;
-  char line[80];
-  char *line_ptr = line;
-  char *command;
-  char *tmpstring;
-  int state = 1;
   
   setvbuf(stdout, (char *)NULL, _IONBF, 0); /* No buffering. */
   
@@ -601,6 +598,28 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
     sgf_initialized = 0;
   }
 
+  do_play_ascii(gameinfo);
+  printf("\nThanks! for playing GNU Go.\n\n");
+
+  /* main() frees the tree and we might have changed it. */
+  *tree = sgftree;
+}
+
+
+void
+do_play_ascii(Gameinfo *gameinfo)
+{
+  int m, num;
+  int sz;
+  float fnum;
+  int passes = 0;  /* two passes and its over */
+  int tmp;
+  char line[80];
+  char *line_ptr = line;
+  char *command;
+  char *tmpstring;
+  int state = 1;
+
   while (state == 1) {
     state = 0;
 
@@ -624,16 +643,23 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
       /* Display game board. */
       if (opt_showboard)
 	ascii_showboard();
-      
+
+#ifndef READLINE
       /* Print the prompt */
       mprintf("%s(%d): ", color_to_string(gameinfo->to_move), movenum + 1);
 
       /* Read a line of input. */
       line_ptr = line;
-      if (!fgets(line, 80, stdin)) {
-	printf("\nThanks! for playing GNU Go.\n\n");
+      if (!fgets(line, 80, stdin))
 	return;
-      }
+#else
+      snprintf(line, 79, "%s(%d): ",
+	       color_to_string(gameinfo->to_move), movenum + 1);
+      if (!(line_ptr = readline(line)))
+	return;
+
+      add_history(line_ptr);
+#endif
 
       while (state == 0
 	     && (command = strtok(line_ptr, ";"), line_ptr = 0, command)) {
@@ -646,7 +672,6 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	case END:
 	case EXIT:
 	case QUIT:
-	  printf("\nThanks! for playing GNU Go.\n\n");
 	  return;
 
 	case HELP:
@@ -895,7 +920,6 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_MOYO;
 	  examine_position(EXAMINE_DRAGONS);
-	  print_moyo();
 	  printmoyo = tmp;
 	  break;
 
@@ -903,7 +927,6 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_TERRITORY;
 	  examine_position(EXAMINE_DRAGONS);
-	  print_moyo();
 	  printmoyo = tmp;
 	  break;
 
@@ -911,7 +934,6 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	  tmp = printmoyo;
 	  printmoyo = PRINTMOYO_AREA;
 	  examine_position(EXAMINE_DRAGONS);
-	  print_moyo();
 	  printmoyo = tmp;
 	  break;
 
@@ -975,6 +997,10 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 	if (passes >= 2)
 	  state = ascii_endgame(gameinfo, 0);
       }
+#ifdef READLINE
+	free(line_ptr);
+#endif
+
     }
 
     sgffile_output(&sgftree);
@@ -991,16 +1017,6 @@ play_ascii(SGFTree *tree, Gameinfo *gameinfo, char *filename, char *until)
 
     gameinfo_clear(gameinfo, board_size, komi);
   }
-
-  printf("\nThanks! for playing GNU Go.\n\n");
-}
-
-void
-play_ascii_emacs(SGFTree *tree, Gameinfo *gameinfo,
-		 char *filename, char *until)
-{
-  emacs = 1;
-  play_ascii(tree, gameinfo, filename, until);
 }
 
 
@@ -1018,7 +1034,7 @@ ascii_endgame(Gameinfo *gameinfo, int reason)
     gnugo_who_wins(gameinfo->computer_player, stdout);
     printf("\nIf you disagree, we may count the game together.\n");
 
-    sgftreeWriteResult(&sgftree, estimate_score(NULL, NULL), 1);
+    sgftreeWriteResult(&sgftree, (white_score + black_score)/2.0, 1);
   }
   else {
     int color = OTHER_COLOR(gameinfo->to_move);
