@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -45,7 +45,6 @@ play_gmp(Gameinfo *gameinfo, int simplified)
   const char *error;
   
   int i, j;
-  int moveval;
   int passes = 0; /* two passes and its over */
   int to_move;  /* who's turn is next ? */
 
@@ -142,7 +141,6 @@ play_gmp(Gameinfo *gameinfo, int simplified)
   while (passes < 2) {
 
     if (to_move == yourcolor) {
-      moveval = 0;
       /* Get opponent's move from gmp client. */
       message = gmp_check(ge, 1, &j, &i, &error);
 
@@ -188,11 +186,11 @@ play_gmp(Gameinfo *gameinfo, int simplified)
     }
     else {
       /* Generate my next move. */
-      moveval = gnugo_genmove(&i, &j, mycolor);
+      float move_value = gnugo_genmove(&i, &j, mycolor, NULL);
       gnugo_play_move(i, j, mycolor);
-      sgffile_add_debuginfo(sgftree.lastnode, moveval);
+      sgffile_add_debuginfo(sgftree.lastnode, move_value);
       
-      if (moveval < 0) {
+      if (gnugo_is_pass(i, j)) {
 	/* pass */
         sgftreeAddPlay(&sgftree, to_move, -1, -1);
 	gmp_sendPass(ge);
@@ -205,7 +203,7 @@ play_gmp(Gameinfo *gameinfo, int simplified)
 	passes = 0;
 	TRACE("\nmy move: %m\n\n", i, j);
       }
-      sgffile_add_debuginfo(sgftree.lastnode, 0);
+      sgffile_add_debuginfo(sgftree.lastnode, 0.0);
       sgffile_output(&sgftree);
     }
     
