@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002 and 2003                         *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -40,7 +40,7 @@ play_solo(Gameinfo *gameinfo, int moves)
 {
   SGFTree sgftree;
   int passes = 0; /* num. consecutive passes */
-  int move_val;
+  float move_value;
   double t1, t2;
   int save_moves = moves;
   int boardsize = gnugo_get_boardsize();
@@ -83,15 +83,15 @@ play_solo(Gameinfo *gameinfo, int moves)
   memset(&totalstats, '\0', sizeof(totalstats));
   while (passes < 2 && --moves >= 0) {
     reset_owl_node_counter();
-    move_val = gnugo_genmove(&i, &j, gameinfo->to_move);
+    move_value = gnugo_genmove(&i, &j, gameinfo->to_move, NULL);
 
     gnugo_play_move(i, j, gameinfo->to_move);
-    sgffile_add_debuginfo(sgftree.lastnode, move_val);
+    sgffile_add_debuginfo(sgftree.lastnode, move_value);
     sgftreeAddPlay(&sgftree, gameinfo->to_move, i, j);
     sgffile_output(&sgftree);
     gameinfo->to_move = OTHER_COLOR(gameinfo->to_move);
 
-    if (move_val < 0) {
+    if (POS(i, j) == PASS_MOVE) {
       ++passes;
       printf("%s(%d): Pass\n", gameinfo->to_move == BLACK ? "Black" : "White",
 	     movenum);
@@ -102,38 +102,33 @@ play_solo(Gameinfo *gameinfo, int moves)
 	      movenum, i, j);
     }
 
-    totalstats.nodes               += stats.nodes;
-    totalstats.position_entered    += stats.position_entered;
-    totalstats.position_hits       += stats.position_hits;
-    totalstats.read_result_entered += stats.read_result_entered;
-    totalstats.hash_collisions     += stats.hash_collisions;
-    total_owl_count                += get_owl_node_counter();
+    totalstats.nodes                    += stats.nodes;
+    totalstats.read_result_entered      += stats.read_result_entered;
+    totalstats.read_result_hits         += stats.read_result_hits;
+    totalstats.trusted_read_result_hits += stats.trusted_read_result_hits;
+    total_owl_count                     += get_owl_node_counter();
   }
   t2 = gg_cputime();
   
   /* Two passes and it's over. (EMPTY == BOTH) */
   gnugo_who_wins(EMPTY, stdout);
 
-  score = gnugo_estimate_score(&upper_bound, &lower_bound);
-  sgfWriteResult(sgftree.root, score, 1);
+  {
+    float score = gnugo_estimate_score(NULL, NULL);
+    sgfWriteResult(sgftree.root, score, 1);
+  }
   sgffile_output(&sgftree);
 
-#if 0
-  if (t2 == t1)
-    printf("%.3f moves played\n", (double) (save_moves-moves));
-  else
-    printf("%.3f moves/sec\n", (save_moves-moves)/(t2-t1));
-#else
   printf("%10d moves played in %0.3f seconds\n", save_moves-moves, t2-t1);
   if (save_moves != moves)
     printf("%10.3f seconds/move\n", (t2-t1)/(save_moves-moves));
+  
   printf("%10d nodes\n", totalstats.nodes);
-  printf("%10d positions entered\n", totalstats.position_entered);
-  printf("%10d position hits\n", totalstats.position_hits);
   printf("%10d read results entered\n", totalstats.read_result_entered);
-  printf("%10d hash collisions\n", totalstats.hash_collisions);
+  printf("%10d read result hits\n", totalstats.read_result_hits);
+  printf("%10d trusted read result hits\n",
+	 totalstats.trusted_read_result_hits);
   printf("%10d owl nodes\n", total_owl_count);
-#endif
 }
 
 
@@ -150,7 +145,7 @@ load_and_analyze_sgf_file(Gameinfo *gameinfo)
   SGFTree sgftree;
   int i, j;
   int next;
-  int move_val;
+  float move_value;
   
   next = gameinfo->to_move;
   sgftree = gameinfo->game_record;
@@ -158,7 +153,7 @@ load_and_analyze_sgf_file(Gameinfo *gameinfo)
   if (metamachine)
     sgffile_begindump(&sgftree);
 
-  move_val = gnugo_genmove(&i, &j, next);
+  move_value = gnugo_genmove(&i, &j, next, NULL);
 
   if (is_pass(POS(i, j)))
     gprintf("%s move: PASS!\n", next == WHITE ? "white (O)" : "black (X)");
@@ -171,7 +166,7 @@ load_and_analyze_sgf_file(Gameinfo *gameinfo)
     gnugo_play_move(i, j, next);
     sgftreeAddPlay(&sgftree, next, i, j);
     sgftreeAddComment(&sgftree, "load and analyze mode");
-    sgffile_add_debuginfo(sgftree.lastnode, move_val);
+    sgffile_add_debuginfo(sgftree.lastnode, move_value);
     sgffile_output(&sgftree);
   }
 }
@@ -194,13 +189,15 @@ void
 load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo, 
 			const char *scoringmode)
 {
-  int i, j, move_val;
+  int move;
+  float move_value;
   char *tempc = NULL;
   char text[250];
   char winner;
   int next;
   int pass = 0;
   int method;
+  float score;
   SGFTree local_tree;
   SGFTree *score_tree = tree;
   
@@ -239,20 +236,20 @@ load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo,
   if (method != ESTIMATE) {
     doing_scoring = 1;
     while (pass < 2) {
-      move_val = genmove_conservative(&i, &j, next);
-      if (move_val >= 0) {
+      move = genmove_conservative(next, &move_value);
+      if (move != PASS_MOVE) {
 	pass = 0;
-	gprintf("%d %s move %m\n", movenum,
-		next == WHITE ? "white (O)" : "black (X)", i, j);
+	gprintf("%d %s move %1m\n", movenum,
+		next == WHITE ? "white (O)" : "black (X)", move);
       }
       else {
 	pass++;
 	gprintf("%d %s move : PASS!\n", movenum, 
 		next == WHITE ? "white (O)" : "black (X)");
       }
-      play_move(POS(i, j), next);
-      sgffile_add_debuginfo(score_tree->lastnode, move_val);
-      sgftreeAddPlay(score_tree, next, i, j);
+      play_move(move, next);
+      sgffile_add_debuginfo(score_tree->lastnode, move_value);
+      sgftreeAddPlay(score_tree, next, I(move), J(move));
       sgffile_output(score_tree);
       next = OTHER_COLOR(next);
     }
@@ -267,7 +264,7 @@ load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo,
      * dragon status is computed. Therefore the call to
      * examine_position().
      */
-    examine_position(next, EXAMINE_ALL);
+    examine_position(EXAMINE_ALL);
     score = gnugo_estimate_score(NULL, NULL);
   }
   
@@ -290,7 +287,7 @@ load_and_score_sgf_file(SGFTree *tree, Gameinfo *gameinfo,
    * what's stored in the game record.
    *
    * FIXME: No comparison is made if the stored result was 0. Wins by
-   *        time or forfeit or not handled either.
+   *        time or forfeit are not handled either.
    *
    * FIXME: Does anybody actually care about this information? Just
    *        removing this piece of code is a tempting alternative.
