@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -163,52 +163,53 @@ replay_node(SGFNode *node, int color_to_replay)
 
   if (color == color_to_replay || color_to_replay == GRAY) {
     /* Get a move from the engine for color. */
-    float move_val = gnugo_genmove(&i, &j, color);
+    int resign;
+    gnugo_genmove(&i, &j, color, &resign);
     /* Now report on how well the computer generated the move. */
     if (i != m || j != n || !quiet) {
       mprintf("Move %d (%C): ", movenum + 1, color);
     
-      if (move_val < 0.0 && !gnugo_is_pass(i, j))
+      if (resign)
 	printf("GNU Go resigns ");
       else {
 	mprintf("GNU Go plays %m ", i, j);
 	if (!gnugo_is_pass(i, j))
-	  printf("(%.2f) ", potential_moves[i][j]);
+	  printf("(%.2f) ", potential_moves[POS(i, j)]);
       }
       mprintf("- Game move %m ", m, n);
-      if (!gnugo_is_pass(m, n) && potential_moves[m][n] > 0.0)
-	printf("(%.2f) ", potential_moves[m][n]);
+      if (!gnugo_is_pass(m, n) && potential_moves[POS(m, n)] > 0.0)
+	printf("(%.2f) ", potential_moves[POS(m, n)]);
       printf("\n");
 
       if (!quiet) {
-	replay_score += (potential_moves[i][j] - potential_moves[m][n]);
-	total_score += potential_moves[i][j];
+	replay_score += (potential_moves[POS(i, j)] - potential_moves[POS(m, n)]);
+	total_score += potential_moves[POS(i, j)];
       }
     }
     if (i != m || j != n) {
-      if (move_val < 0.0 && !gnugo_is_pass(i, j))
+      if (resign)
 	gg_snprintf(buf, 127, "GNU Go resigns - Game move %s (%.2f)",
 		    location_to_string(POS(m, n)),
 		    gnugo_is_pass(m, n)
-		    && potential_moves[m][n] < 0.0 ?
-			0 : potential_moves[m][n]);
+		    && potential_moves[POS(m, n)] < 0.0 ?
+			0 : potential_moves[POS(m, n)]);
       else {      
 	gg_snprintf(buf, 127, "GNU Go plays %s (%.2f) - Game move %s (%.2f)",
 		    location_to_string(POS(i, j)),
-		    gnugo_is_pass(i, j) ? 0 : potential_moves[i][j],
+		    gnugo_is_pass(i, j) ? 0 : potential_moves[POS(i, j)],
 		    location_to_string(POS(m, n)),
 		    gnugo_is_pass(m, n)
-		    && potential_moves[m][n] < 0.0 ?
-			0 : potential_moves[m][n]);
+		    && potential_moves[POS(m, n)] < 0.0 ?
+			0 : potential_moves[POS(m, n)]);
 	sgfCircle(node, i, j);
       }
     }
     else
       gg_snprintf(buf, 127, "GNU Go plays the same move %s (%.2f)",
                  location_to_string(POS(i, j)),
-                 gnugo_is_pass(i, j) ? 0 : potential_moves[i][j]);
+                 gnugo_is_pass(i, j) ? 0 : potential_moves[POS(i, j)]);
     sgfAddComment(node, buf);
-    sgffile_add_debuginfo(node, 0);
+    sgffile_add_debuginfo(node, 0.0);
   }
 
   /* Finally, do play the move from the file. */
