@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -416,8 +416,8 @@ print_c_dfa(FILE *of, const char *name, dfa_t *pdfa)
     exit(EXIT_FAILURE);
   }
 
-  assert(dfa_minmax_delta(pdfa, -1, 1) > 0);
-  if (dfa_minmax_delta(pdfa, -1, 0)  > 65535) {
+  assert(dfa_minmax_delta(pdfa, -1, 1) > -32768);
+  if (dfa_minmax_delta(pdfa, -1, 0)  > 32768) {
     fprintf(of, "#error too many states");
     fprintf(stderr, "Error: The dfa states are too disperse. Can't fit delta into a short.\n");
     exit(EXIT_FAILURE);
@@ -436,16 +436,15 @@ print_c_dfa(FILE *of, const char *name, dfa_t *pdfa)
 	  name, pdfa->last_state + 1);
   for (i = 0; i != pdfa->last_state + 1; i++) {
     int j;
-    fprintf(of, "{%d,", pdfa->states[i].att);
-    fprintf(of, "{");
+    fprintf(of, "{{");
     for (j = 0; j < 4; j++) {
       int n = pdfa->states[i].next[j];
-      assert((n == 0) || ((n - i > 0) && (n - i < 65535)));
+      assert((n == 0) || (abs(n - i) < 32768));
       fprintf(of, "%d", n ? n - i : 0);
       if (j != 3)
         fprintf(of, ",");
     }
-    fprintf(of, "}},%s", ((i+1)%3 ? "\t" : "\n"));
+    fprintf(of, "}, %d},%s", pdfa->states[i].att, ((i+1)%3 ? "\t" : "\n"));
   }
   fprintf(of, "};\n\n");
 
@@ -785,7 +784,8 @@ dfa_end(void)
  */
 
 int 
-dfa_minmax_delta(dfa_t *pdfa, int next_index, int isMin) {
+dfa_minmax_delta(dfa_t *pdfa, int next_index, int isMin)
+{
 
   int ret, i, j;
   assert(next_index <= 3);
@@ -816,11 +816,11 @@ dfa_minmax_delta(dfa_t *pdfa, int next_index, int isMin) {
   return ret;
 }
 
+#define DFA_ALIGN 	2
 
 /*
  * Re-orders DFA into a canonical form, which does a half-hearted 
- * attempt to reduce the size of jumps for all states entries, and
- * guarantees the jumps are all forward-only.
+ * attempt to reduce the size of jumps for all states entries.
  */
 void
 dfa_shuffle(dfa_t *pdfa)
@@ -852,11 +852,13 @@ dfa_shuffle(dfa_t *pdfa)
     for (i = 0; i < q1p; i++) {
       for (j = 0; j < 4; j++) {
         int n = pdfa->states[queue1[i]].next[j];
-        if (n && !state_to[n]) {
+	/* next_new_state = DFA_ALIGN * ((next_new_state-1) / DFA_ALIGN) + 1;*/
+        while (n && !state_to[n]) {
           state_to[n] = next_new_state;
           state_from[next_new_state] = n;
           next_new_state++;
           queue2[q2p++] = n;
+	  n = pdfa->states[n].next[0];
         }
       }
     }
@@ -887,39 +889,53 @@ dfa_shuffle(dfa_t *pdfa)
 /* Calculate the maximal number of patterns matched at one point for
  * one transformation.  Multiplying this number by 8 gives an upper
  * bound for the total number of matched patterns for all
- * transformation.  The DFA must be shuffled before calling this
- * function (to ensure that all jumps go forward).
+ * transformation.
  */
 int
 dfa_calculate_max_matched_patterns(dfa_t *pdfa)
 {
   int total_max = 0;
   int *state_max = calloc(pdfa->last_state + 1, sizeof(int));
-  int k;
+  char *queued = calloc(pdfa->last_state + 1, sizeof(char));
+  int *queue = malloc(pdfa->last_state * sizeof(int));
+  int queue_start = 0;
+  int queue_end = 1;
 
-  for (k = 1; k <= pdfa->last_state; k++) {
-    int i;
+  queue[0] = 1;
+  while (queue_start < queue_end) {
+    int state = queue[queue_start++];
+    int k;
 
     /* Increment maximal number of matched patterns for each pattern
-     * matched at state `k'.
+     * matched at current `state'.
      */
-    for (i = pdfa->states[k].att; i; i = pdfa->indexes[i].next)
-      state_max[k]++;
+    for (k = pdfa->states[state].att; k; k = pdfa->indexes[k].next)
+      state_max[state]++;
 
-    if (total_max < state_max[k])
-      total_max = state_max[k];
+    if (total_max < state_max[state])
+      total_max = state_max[state];
+
+    for (k = 0; k < 4; k++) {
+      int next = pdfa->states[state].next[k];
 
-    for (i = 0; i < 4; i++) {
-      int next = pdfa->states[k].next[i];
       if (next != 0) {
-       assert(next > k);
+	if (!queued[next]) {
+	  queue[queue_end++] = next;
+	  queued[next] = 1;
+	}
 
-       if (state_max[next] < state_max[k])
-         state_max[next] = state_max[k];
+	if (state_max[next] < state_max[state])
+	  state_max[next] = state_max[state];
       }
     }
   }
 
+  assert(queue_end == pdfa->last_state);
+
+  free(state_max);
+  free(queued);
+  free(queue);
+
   return total_max;
 }
 
@@ -952,7 +968,6 @@ dfa_finalize(dfa_t *pdfa)
   compactify_att(pdfa);
 }
 
-
 /*
  * Add a new string with attribute att_val into the dfa.
  * if the new size of the dfa respect some size conditions
