@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -46,11 +46,15 @@
 #define _NO_PROTO
 #endif
 
-/* =================================================================== */
-/*  Configure option that has been removed from the stable version.    */
-/* =================================================================== */
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#else
+#define GRID_OPT 0
+#endif
 
-#define GRID_OPT 1
+#ifndef GRID_OPT
+#error GRID_OPT should be defined as 0, 1 or 2
+#endif
 
 
 /* Include support for pattern profiling. May be turned off in stable
@@ -349,7 +353,6 @@ void init_tree_oracle(void);
 
 /* pattern arrays themselves */
 extern struct pattern_db pat_db;
-extern struct corner_db  joseki_db;
 extern struct pattern_db aa_attackpat_db;
 extern struct pattern_db owl_attackpat_db;
 extern struct pattern_db owl_vital_apat_db;
@@ -364,6 +367,8 @@ extern struct pattern_db fusekipat_db;
 extern struct pattern_db handipat_db;
 extern struct pattern_db oracle_db;
 
+extern struct corner_db joseki_db;
+
 extern struct fullboard_pattern fuseki19[];
 extern struct fullboard_pattern fuseki13[];
 extern struct fullboard_pattern fuseki9[];
