@@ -2,7 +2,7 @@
  * This is GNU Go, a Go program. Contact gnugo@gnu.org, or see       *
  * http://www.gnu.org/software/gnugo/ for more information.          *
  *                                                                   *
- * Copyright 1999, 2000, 2001, 2002, 2003 and 2004                   *
+ * Copyright 1999, 2000, 2001, 2002, 2003, 2004 and 2005             *
  * by the Free Software Foundation.                                  *
  *                                                                   *
  * This program is free software; you can redistribute it and/or     *
@@ -16,7 +16,7 @@
  *                                                                   *
  * You should have received a copy of the GNU General Public         *
  * License along with this program; if not, write to the Free        *
- * Software Foundation, Inc., 59 Temple Place - Suite 330,           *
+ * Software Foundation, Inc., 51 Franklin Street, Fifth Floor,       *
  * Boston, MA 02111, USA.                                            *
 \* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
 
@@ -1149,6 +1149,7 @@ readsgffilefuseki(const char *filename, int moves_per_game)
 
   if (sgferr) {
     fprintf(stderr, "Parse error: %s at position %d\n", sgferr, sgferrpos);
+    sgfFreeNode(root);
     return NULL;
   }
 
@@ -1179,6 +1180,7 @@ readsgffilefuseki(const char *filename, int moves_per_game)
 /*
  * Wrapper around readsgf which reads from a file rather than a string.
  * Returns NULL if file will not open, or some other parsing error.
+ * Filename "-" means read from stdin, and leave it open when done.
  */
 
 SGFNode *
@@ -1199,10 +1201,12 @@ readsgffile(const char *filename)
   nexttoken();
   gametree(&root, NULL, LAX_SGF);
 
-  fclose(sgffile);
+  if (sgffile != stdin)
+    fclose(sgffile);
 
   if (sgferr) {
     fprintf(stderr, "Parse error: %s at position %d\n", sgferr, sgferrpos);
+    sgfFreeNode(root);
     return NULL;
   }
 
@@ -1504,7 +1508,8 @@ writesgf(SGFNode *root, const char *filename)
 
   sgf_column = 0;
   unparse_game(outfile, root, 1);
-  fclose(outfile);
+  if (outfile != stdout)
+    fclose(outfile);
   
   /* Remove "printed" marks so that the tree can be written multiple
    * times.
